import sys
import os
import multiprocessing
try:
    import cPickle as pickle
except:
    import pickle

import traceback
import time
import logging

import h5py
import numpy as np
import pandas as pd

from sklearn.utils import resample
from sklearn.preprocessing import Imputer, StandardScaler, normalize
from sklearn.ensemble import RandomForestClassifier
from sklearn import cross_validation as cv
from sklearn import metrics
try:
    from . import hmm
except:
    print('Problem import hmm module - presumably do not have pomegranate insalled?')

from .h5loader import H5File

def load_classifier(filepath):
    f = open(filepath, 'rb')
    clf = pickle.load(f)
    return clf

class FeaturePreProcesser():
    def __init__(self):
        pass

    def fit(self,X):
        self.imputer = Imputer(missing_values='NaN', strategy='mean', axis=0)
        self.imputer.fit(X)
        X = self.imputer.transform(X)

        self.std_scaler = StandardScaler()
        self.std_scaler.fit(X)

    def fit_transform(self, X):
        self.imputer = Imputer(missing_values='NaN', strategy='mean', axis=0)
        self.imputer.fit(X)
        X = self.imputer.transform(X)

        self.std_scaler = StandardScaler()
        self.std_scaler.fit(X)
        X = self.std_scaler.transform(X)

        return X
    def transform(self, X):
        X = self.imputer.transform(X)
        X = self.std_scaler.transform(X)
        return X

class Classifier():
    ''' we drop the actual data... so this is a lot smaller than the library used to train it '''
    def __init__(self, library_path):
        with h5py.File(library_path, 'r+') as f:
            self.keys = list(f.keys())
            self.keys = np.random.permutation(self.keys)
            self.tids = [f[key].attrs['tid'] for key in self.keys]
            self.fname_array = np.vstack([np.vstack([fname for i in range(f[fname+'/features'].shape[0])]) for fname in self.keys])
            self.features = np.vstack( f[name+'/features'] for name in self.keys)
            self.labels   = np.hstack( f[name+'/labels'] for name in self.keys)
            self.feature_names = f[self.keys[0]].attrs['col_names'].astype(str)

        self.cleaner = FeaturePreProcesser()
        self.cleaner.fit(self.features)
        self.features = self.cleaner.transform(self.features)

    def save(self, fname = 'pickled_clf' ):
        fname = fname if fname.endswith('.p') else fname+'.p'
        f = open(fname,'wb')
        pickle.dump(self,f)

    def load(self, fname):
        print('This is a function of the module, not a particular classifier! (Naturally will not exist)')
        return 0

    def train(self, downsample_bl_by_x = 3,
                    upsample_seizure_by_x = 1,
                    ntrees=800, n_cores = -1,
                    n_emission_prob_cvfolds = 3):
        
        if upsample_seizure_by_x != 1:
            print('Warning: you are upsampling minority class - oob error cannot be trusted' )
        
        counts = pd.Series(np.ravel(self.labels[:])).value_counts().values
        target_resample = (int(counts[0]/self.downsample_bl_factor),counts[1]*self.upsample_seizure_factor)

        logging.info('Training classifier: ')
        logging.info('Resampling training - upsampling seizure seconds, downsampling baseline periods...')
        res_y, res_x = self.resample_training_dataset(self.labels, self.features,
                                                      sizes = target_resample)

        logging.info('Training Random Forest on resampled data')
        self.clf =  RandomForestClassifier(n_jobs=n_cores, n_estimators= ntrees, oob_score=True, bootstrap=True)
        self.clf.fit(res_x, np.ravel(res_y))

        logging.info('Getting HMM params')
        self.make_hmm_model(n_emission_prob_cvfolds) # uses normal data, you should pass downsampling params?

        print ('********* oob results on resampled data *******')
        self.oob_preds = np.round(self.clf.oob_decision_function_[:,1])
        print('ROC_AUC score: '+str(metrics.roc_auc_score(np.ravel(res_y), self.clf.oob_decision_function_[:,1])))
        print('Recall: '+str(metrics.recall_score(np.ravel(res_y), self.oob_preds)))
        print('F1: '+str(metrics.f1_score(np.ravel(res_y), self.oob_preds)))
        print(metrics.classification_report(np.ravel(res_y),self.oob_preds))

        self.feature_weightings = sorted(zip(self.clf.feature_importances_, self.feature_names),reverse = True)

    def make_hmm_model(self, n_emission_prob_folds = 3):
        # how to get these bad boys...
        self.emission_probs = self.get_cross_validation_emission_probs(self.features,
                                                                       self.labels,
                                                                       nfolds = n_emission_prob_folds)
        print('Emission probs')
        print (self.emission_probs)
        self.transition_probs = hmm.get_state_transition_probs(self.labels)
        print ('Transition probs')
        print (self.transition_probs)
        self.hm_model = hmm.make_hmm_model(self.emission_probs,self.transition_probs)
        
    def get_cross_validation_emission_probs(self, X, y, nfolds = 3):
        '''
        - X has been imputed and cleaned etc...
        - Also, we are using default RF, should nest some
        tuning within it! - too long

        Returns:
            - HMM model emission probs.
            - ToDO: Error (though HMM?)
            - ToDO: best RF parameters...
            - ToDO: potentially threshold
            - ToDO: best resampling?
        '''
        logging.info('Starting stratified cross validation with '+ str(nfolds)+ ' folds!' )

        emission_matrices_list = [] # to hold the result from the k fold

        if len(y.shape) != 1: # ravel labels if they need it
            y = np.ravel(y)

        self.printProgress(0,nfolds, prefix = 'Getting hmm emission probs:', suffix = 'Complete', barLength = 30)
        fold = 1

        skf = cv.StratifiedKFold(y, nfolds, random_state= 7)
        precision, recall, f1 = [],[],[]
        oob_precision, oob_recall, oob_f1 = [],[],[]
        for train_ix,test_ix in skf:

            # index for the k fold
            X_train = X[train_ix,:]
            y_train = y[train_ix]
            X_test  = X[test_ix,:]
            y_test  = y[test_ix]

            # work out desired resampling - this needs to be inheritied
            y_counts = pd.Series(y_train).value_counts().values
            target_resample = (y_counts[1]*50,y_counts[1])
            samp_y, samp_x = self.resample_training_dataset(y_train, X_train, sizes = target_resample)

            # train clf on resampled
            rf = RandomForestClassifier(n_jobs=-1, n_estimators= 500, oob_score=True, bootstrap=True)
            rf.fit(samp_x, np.ravel(samp_y))

            # see how well it did
            train_emitts = rf.predict(X_train)
            test_emitts  = rf.predict(X_test)

            # this is the test set
            p_score = metrics.precision_score(y_test, test_emitts)
            precision.append(p_score)
            recall_score = metrics.recall_score(y_test, test_emitts)
            recall.append(recall_score)
            f1_score = metrics.f1_score(y_test, test_emitts)
            f1.append(f1_score)


            # this is the oob on the resampled
            oob_preds = np.round(rf.oob_decision_function_[:,1]) # round the 1 column for 1 issezi. 0 is baseline
            oob_p_score = metrics.precision_score(np.ravel(samp_y), oob_preds)
            oob_precision.append(oob_p_score)
            oob_recall_score = metrics.recall_score(np.ravel(samp_y), oob_preds)
            oob_recall.append(oob_recall_score)
            oob_f1_score = metrics.f1_score(np.ravel(samp_y), oob_preds)
            oob_f1.append(oob_f1_score)


            binary_states = np.where(y_test==0,0,1) # this is for when multiple labels
            emission_matrix = hmm.get_state_emission_probs(test_emitts, binary_states)
            emission_matrices_list.append(emission_matrix)
            self.printProgress(fold,nfolds, prefix = 'Getting hmm emission probs:', suffix = 'Complete', barLength = 30)
            fold += 1

        logging.info('Precision: ' + str(precision))
        logging.info('Recall: ' + str(recall))
        logging.info('F1: ' + str(f1))

        logging.info('OOB is on the resampled data:')
        logging.info('oob_Precision: ' + str(oob_precision))
        logging.info('oob_Recall: ' + str(oob_recall))
        logging.info('oob_F1: ' + str(oob_f1))

        #print( 'Mean precision'+ str(np.mean(precision)) )
        #print( 'Mean recall: '+ str(np.mean(recall)) )
        #print( 'Mean f1: '+str(np.mean(f1)) )

        ems = np.stack(emission_matrices_list, axis = 2)
        mean_emitt_matrix = np.mean(ems, axis = 2)
        return mean_emitt_matrix

    def estimate_clf_error(self, nfolds = 3):
        #self.printProgress(0,nfolds, prefix = 'Cross validation:', suffix = 'Complete', barLength = 50)
        #fold = 1
        print ('Running '+str(nfolds)+'-fold cross validation to estimate classifier performance:')
        skf = cv.StratifiedKFold(np.ravel(self.labels), nfolds, random_state= 7)
        precision, recall, f1 = [],[],[]
        map_precision, map_recall, map_f1 = [],[],[]

        for train_ix,test_ix in skf:
            X_train = self.features[train_ix,:]
            y_train = self.labels[train_ix]
            X_test  = self.features[test_ix,:]
            y_test  = self.labels[test_ix]

            y_counts = pd.Series(y_train).value_counts().values
            target_resample = (y_counts[1]*50,y_counts[1]) #ie. 50* seizures n
            samp_y, samp_x = self.resample_training_dataset(y_train, X_train, sizes = target_resample)

            print('running balanced, undersampling only')
            rf = RandomForestClassifier(n_jobs=-1, n_estimators= 500,
                                        oob_score=True, bootstrap=True,
                                        class_weight= 'balanced')
            rf.fit(samp_x, np.ravel(samp_y))
            #rf.fit(X_train, np.ravel(y_train))

            # now you need the hmm params! nest this shit up
            train_emission_probs = self.get_cross_validation_emission_probs(X_train, y_train,nfolds = 4)
            train_transition_probs = hmm.get_state_transition_probs(y_train)
            train_hm_model = hmm.make_hmm_model(train_emission_probs,train_transition_probs)

            # predict the y_test with the rf trained on the resampled data:
            test_emissions = rf.predict(X_test)
            #viterbi_decoded = train_hm_model.predict(test_emissions, algorithm='viterbi')[1:]
            logp, path = train_hm_model.viterbi(test_emissions)
            viterbi_decoded = np.array([int(state.name) for idx, state in path[1:]])

            p_score = metrics.precision_score(y_test, viterbi_decoded)
            precision.append(p_score)
            recall_score = metrics.recall_score(y_test, viterbi_decoded)
            recall.append(recall_score)
            f1_score = metrics.f1_score(y_test, viterbi_decoded)
            f1.append(f1_score)
            print ('vit Precision: '+ str(p_score))
            print ('vit Recall: '+ str(recall_score))
            print ('vit F1: '+ str(f1_score))

            try:
                # run forward backward as well
                fb = train_hm_model.predict(test_emissions, algorithm='map')
                p_score = metrics.precision_score(y_test, fb)
                map_precision.append(p_score)
                recall_score = metrics.recall_score(y_test, fb)
                map_recall.append(recall_score)
                f1_score = metrics.f1_score(y_test, fb)
                map_f1.append(f1_score)
                #print ('fb Precision: '+ str(p_score))
                #print ('fb Recall: '+ str(recall_score))
                #print ('fb F1: '+ str(f1_score))
                fb = True
            except:
                fb = False
                pass


        clf_precision = np.mean(precision)
        clf_recall = np.mean(recall)
        clf_f1 = np.mean(f1)

        print ('Mean vit precision: '+str(clf_precision))
        print ('Mean vit recall:    '+ str(clf_recall))
        print ('Mean vit f1:        '+ str(clf_f1))

        if fb:
            map_clf_precision = np.mean(map_precision)
            map_clf_recall = np.mean(map_recall)
            map_clf_f1 = np.mean(map_f1)

            print ('Mean fb precision: '+str(map_clf_precision))
            print ('Mean fb recall:    '+ str(map_clf_recall))
            print ('Mean fb f1:        '+ str(map_clf_f1))

    def resample_training_dataset(self, labels, feature_array, sizes = (5000,500)):
        """
        Inputs:
            - labels
            - features
            - sizes: tuple, for each class (0,1,etc)m the number of training chunks you want.
            i.e for 500 seizures, 5000 baseline, sizes = (5000, 500), as 0 is baseline, 1 is Seizure
        Takes labels and features an

        WARNING: Up-sampling target class prevents random forest oob from being accurate.
        """
        if len (labels.shape) == 1:
            labels = labels[:, None]

        resampled_labels = []
        resampled_features = []
        for i,label in enumerate(np.unique(labels.astype('int'))):
            class_inds = np.where(labels==label)[0]

            class_labels = labels[class_inds]
            class_features = feature_array[class_inds,:]

            if class_features.shape[0] < sizes[i]: # need to oversample
                class_features_duplicated = np.vstack([class_features for i in range(int(sizes[i]/class_features.shape[0]))])
                class_labels_duplicated  = np.vstack([class_labels for i in range(int(sizes[i]/class_labels.shape[0]))])
                n_extra_needed = sizes[i] - class_labels_duplicated.shape[0]
                extra_features = resample(class_features, n_samples =  n_extra_needed,random_state = 7, replace = False)
                extra_labels = resample(class_labels, n_samples =  n_extra_needed,random_state = 7, replace = False)

                boot_array  = np.vstack([class_features_duplicated,extra_features])
                boot_labels = np.vstack([class_labels_duplicated,extra_labels])

            elif class_features.shape[0] > sizes[i]: # need to undersample
                boot_array  = resample(class_features, n_samples =  sizes[i],random_state = 7, replace = False)
                boot_labels = resample(class_labels,   n_samples =  sizes[i],random_state = 7, replace = False)

            elif class_features.shape[0] == sizes[i]:
                logging.debug('label '+str(label)+ ' had exact n as sample, doing nothing!')
                boot_array  = class_features
                boot_labels = class_labels
            else:
                print(class_features.shape[0], sizes[i])
                print ('fuckup')
            resampled_features.append(boot_array)
            resampled_labels.append(boot_labels)
        # stack both up...
        resampled_labels = np.vstack(resampled_labels)
        resampled_features = np.vstack(resampled_features)

        logging.debug('Original label counts: '+str(pd.Series(labels[:,0]).value_counts()))
        logging.debug('Resampled label counts: '+str(pd.Series(resampled_labels[:,0]).value_counts()))

        return resampled_labels, resampled_features


    def tune_hyperparameters(self):
        pass
        # just a placeholderfor now - code is in ipython notebook
        # we want to tune class weights?
        # also depth, leaves and n features?


    def predict_dir(self, prediction_dir, excel_sheet = 'clf_predictions.csv'):
        files_to_predict = os.listdir(prediction_dir)
        print('Looking through '+ str(len(files_to_predict)) + ' file for seizures:')
        for fname in files_to_predict:
            fpath = os.path.join(prediction_dir,fname)

            try:
                with h5py.File(fpath, 'r+') as f:
                    group = f[list(f.keys())[0]]
                    tid = group[list(group.keys())[0]]
                    pred_features = tid['features'][:]

                logging.info(str(fname) + ' now predicting!: ')
                pred_features = self.cleaner.transform(pred_features)
                pred_y_emitts = self.clf.predict(pred_features)
                logp, path = self.hm_model.viterbi(pred_y_emitts)
                vit_decoded_y = np.array([int(state.name) for idx, state in path[1:]])

                if sum(vit_decoded_y):
                    name_array = np.array([fname for i in range(vit_decoded_y.shape[0])])
                    #print (name_array.shape)
                    pred_sheet = self.make_excel_spreadsheet([name_array,vit_decoded_y])
                    #print(pred_sheet.head())
                    if not os.path.exists(excel_sheet):
                        pred_sheet.to_csv(excel_sheet,index = False)
                    else:
                        with open(excel_sheet, 'a') as f:
                            pred_sheet.to_csv(f, header=False, index = False)


            except KeyError:
                logging.error(str(fname) + ' did not contain any features! Skipping')

            else:
                pass
                #print ('no seizures')

        print('Re - ordering spreadsheet by date')
        self.reorder_prediction_csv(excel_sheet)
        print ('Done')

    def reorder_prediction_csv(self, csv_path):
        df = pd.read_csv(csv_path)
        datetimes = []
        for string in df.Filename:
            ymd = string.split('_')[1]
            h = string.split('_')[2]
            m = string.split('_')[3]
            x = ymd+h+m
            datetimes.append(pd.to_datetime(x, format = '%Y-%m-%d%H%M'))
        df['datetime'] = pd.Series(datetimes)
        reordered_df = df.sort_values(by=['datetime', 'Start']).drop('datetime', axis = 1)
        reordered_df.to_csv(csv_path)

    def make_excel_spreadsheet(self, to_stack = [], columns_list = ['Name', 'Pred'], verbose = False):
        '''
        to_stack list:
            - first needs to be the name array
            - second needs to be the predictions
        '''
        for i,array in enumerate(to_stack):
            if len(array.shape) == 1:
                to_stack[i] = array[:,None]
        data = np.hstack([to_stack[0],to_stack[1].astype(int)])
        df = pd.DataFrame(data, columns = columns_list)

        # Make time index...
        df['f_index'] = df.groupby(by = columns_list[0]).cumcount()
        # assuming 1 hour per filename, first assert all files have same
        # number of
        x = df[str(columns_list[0])].value_counts()
        n_chunks = x.max()
        try:
            assert x.isin([n_chunks]).all()
        except:
            if verbose:
                print('Warning: Files not all the same length \n'+str(x))
            else:
                print('Warning: Files not all the same length, run again with verbose flag True for more detail')
        sec_per_chunk = 3600/n_chunks
        df['start_time'] = df['f_index']*sec_per_chunk
        df['end_time'] = (df['f_index']+1)*sec_per_chunk

        # okay, now find the seizures!
        seizure_indexes = df.groupby(columns_list[1]).indices['1']
        seizures_idx_tups = []
        start = None
        for i,t  in enumerate(seizure_indexes):
            if start is None:
                start = t
            try:
                if seizure_indexes[i+1] - t != 1:
                    end = t
                    seizures_idx_tups.append((start,end))
                    start = None
            except IndexError:
                end = t
                seizures_idx_tups.append((start,end))
                start = None

        # make the Dataframe for predicted seizures
        df_rows = []
        for tup in seizures_idx_tups:
            name = df.ix[tup[0],columns_list[0]]
            start_time = df.ix[tup[0],'start_time']
            end_time = df.ix[tup[1],'end_time']
            row = pd.Series([name,start_time,end_time],
                            index = ['Filename','Start', 'End'])
            df_rows.append(row)
        excel_sheet = pd.DataFrame(df_rows)
        return excel_sheet

    def printProgress (self, iteration, total, prefix = '', suffix = '', decimals = 2, barLength = 100):
        """
        Call in a loop to create terminal progress bar
        @params:
            iteration   - Required  : current iteration (Int)
            total       - Required  : total iterations (Int)
            prefix      - Optional  : prefix string (Str)
            suffix      - Optional  : suffix string (Str)
            decimals    - Optional  : number of decimals in percent complete (Int)
            barLength   - Optional  : character length of bar (Int)
        """
        filledLength    = int(round(barLength * iteration / float(total)))
        percents        = round(100.00 * (iteration / float(total)), decimals)
        bar             = '█' * filledLength + '-' * (barLength - filledLength)
        sys.stdout.write('\r%s |%s| %s%s %s' % (prefix, bar, percents, '%', suffix)),
        sys.stdout.flush()
        if iteration == total:
            sys.stdout.write('\n')
            sys.stdout.flush()


# -*- coding: utf-8 -*-
"""
Plans:

** Dont understand the difference bewtween views, layout etc, documentation seems very hard to find
Lets just make miminum viable thing - scrolling and ability to load and select

 - left right arrow
 - space bar scrolling
 - nested tids/ seizure datasets
 - fft
 - animal video space
 - larger bottom
 - disable zooms on smaller
 - be able to navigate between files...

 Other examples...:
  - scrollling
  - nested
  - dialogue boxes
  - graph view...


"""

#import initExample ## Add path to library (just for examples; you do not need this)
import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui
import numpy as np
import h5py
import sys, os

class HDF5Plot(pg.PlotCurveItem):
    """
    Create a subclass of PlotCurveItem for displaying a very large
    data set from an HDF5 file that does not neccesarilly fit in memory.

    The basic approach is to override PlotCurveItem.viewRangeChanged such that it
    reads only the portion of the HDF5 data that is necessary to display the visible
    portion of the data. This is further downsampled to reduce the number of samples
    being displayed.

    A more clever implementation of this class would employ some kind of caching
    to avoid re-reading the entire visible waveform at every update.
    """
    def __init__(self, downsample_limit = 20000,viewbox = None, *args, **kwds):
        " TODO what are the args and kwds for PlotCurveItem class?"
        self.hdf5 = None
        self.time = None
        self.fs = None
        self.vb = viewbox
        self.limit = downsample_limit # maximum number of samples to be plotted, 10000 orginally
        pg.PlotCurveItem.__init__(self, *args, **kwds)


    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Escape:
            self.close()
        else:
            print(key)

    def setHDF5(self, data, time, fs):
        self.hdf5 = data
        self.time = time
        self.fs = fs
        #print ( self.hdf5.shape, self.time.shape)
        self.updateHDF5Plot()

    def viewRangeChanged(self):
        self.updateHDF5Plot()

    def updateHDF5Plot(self):
        if self.hdf5 is None:
            self.setData([])
            return

        #vb = self.getViewBox()
        #if vb is None:
        #    return  # no ViewBox yet

        # Determine what data range must be read from HDF5
        xrange = [i*self.fs for i in self.vb.viewRange()[0]]
        start = max(0,int(xrange[0])-1)
        stop = min(len(self.hdf5), int(xrange[1]+2))

        # Decide by how much we should downsample
        ds = int((stop-start) / self.limit) + 1
        if ds == 1:
            # Small enough to display with no intervention.
            visible_y = self.hdf5[start:stop]
            visible_x = self.time[start:stop]
            scale = 1
        else:
            # Here convert data into a down-sampled array suitable for visualizing.
            # Must do this piecewise to limit memory usage.
            samples = 1 + ((stop-start) // ds)
            visible_y = np.zeros(samples*2, dtype=self.hdf5.dtype)
            visible_x = np.zeros(samples*2, dtype=self.time.dtype)
            sourcePtr = start
            targetPtr = 0

            # read data in chunks of ~1M samples
            chunkSize = (1000000//ds) * ds
            while sourcePtr < stop-1:

                chunk = self.hdf5[sourcePtr:min(stop,sourcePtr+chunkSize)]
                chunk_x = self.time[sourcePtr:min(stop,sourcePtr+chunkSize)]
                sourcePtr += len(chunk)
                #print(chunk.shape, chunk_x.shape)

                # reshape chunk to be integral multiple of ds
                chunk = chunk[:(len(chunk)//ds) * ds].reshape(len(chunk)//ds, ds)
                chunk_x = chunk_x[:(len(chunk_x)//ds) * ds].reshape(len(chunk_x)//ds, ds)

                # compute max and min
                #chunkMax = chunk.max(axis=1)
                #chunkMin = chunk.min(axis=1)

                mx_inds = np.argmax(chunk, axis=1)
                mi_inds = np.argmin(chunk, axis=1)
                row_inds = np.arange(chunk.shape[0])

                chunkMax = chunk[row_inds, mx_inds]
                chunkMin = chunk[row_inds, mi_inds]
                chunkMax_x = chunk_x[row_inds, mx_inds]
                chunkMin_x = chunk_x[row_inds, mi_inds]

                # interleave min and max into plot data to preserve envelope shape
                visible_y[targetPtr:targetPtr+chunk.shape[0]*2:2] = chunkMin
                visible_y[1+targetPtr:1+targetPtr+chunk.shape[0]*2:2] = chunkMax
                visible_x[targetPtr:targetPtr+chunk_x.shape[0]*2:2] = chunkMin_x
                visible_x[1+targetPtr:1+targetPtr+chunk_x.shape[0]*2:2] = chunkMax_x

                targetPtr += chunk.shape[0]*2

            visible_x = visible_x[:targetPtr]
            visible_y = visible_y[:targetPtr]
            #print('**** now downsampling')
            #print(visible_y.shape, visible_x.shape)
            scale = ds * 0.5

        # TODO: setPos, scale, resetTransform methods... scale?
        self.setData(visible_x, visible_y) # update the plot
        #self.setPos(start, 0) # shift to match starting index ### Had comment out to stop it breaking... when limit is >0?!
        self.resetTransform()
        #self.scale(scale, 1)  # scale to match downsampling

def load_library(path):
    "load seizure library files"
    pass

def load_ndf(path):
    pass

def load_h5(path):
    "flat h5 for predictions"
    pass

def createFile(finalSize=2000000000):
    """Create a large HDF5 data file for testing.
    Data consists of 1M random samples tiled through the end of the array.
    """

    chunk = np.random.normal(size=1000000).astype(np.float32)

    f = h5py.File('test.hdf5', 'w')
    f.create_dataset('data', data=chunk, chunks=True, maxshape=(None,))
    data = f['data']

    nChunks = finalSize // (chunk.size * chunk.itemsize)
    with pg.ProgressDialog("Generating test.hdf5...", 0, nChunks) as dlg:
        for i in range(nChunks):
            newshape = [data.shape[0] + chunk.shape[0]]
            data.resize(newshape)
            data[-chunk.shape[0]:] = chunk
            dlg += 1
            if dlg.wasCanceled():
                f.close()
                os.remove('test.hdf5')
                sys.exit()
        dlg += 1
    f.close()


fileName = '/Users/jonathan/Dropbox/gui_pyqt_dev/gl_library_for_vannila_features.h5'
f = h5py.File(fileName, 'r')


i = 0
datasets = [f[key] for key in list(f.keys())]
data = datasets[i]['data'][:]  # check - dont need to slice here? or do i?
# think i have broken the larger than memory part of this...
flat_data = np.ravel(data, order = 'C')
fs = datasets[i].attrs['fs']
time = np.arange(0, 3600*fs)/fs# assuming it is an hour
time = np.reshape(time, newshape=(data.shape), order = 'C') # should be same shape!
flat_time = np.ravel(time, order = 'C')


#pg.mkQApp()
app = QtGui.QApplication([])
#view = pg.GraphicsView()
#win = pg.GraphicsLayout()
#view.setCentralItem(win)

win = pg.GraphicsWindow()

win.resize(1000,500)
win.setWindowTitle('PyECoG ')


#bx2 = win.addViewBox(row = 1, col=1, colspan = 3)
plt1 = win.addPlot(row=1, col=1,colspan = 3, title = 'Overview ... ')
plt1.enableAutoRange(False, False)
plt1.setXRange(0, 3600)
plt1.setMouseEnabled(x = False, y = True)
bx1 = plt1.getViewBox()
curve1 = HDF5Plot(parent = plt1, viewbox = bx1)
curve1.setHDF5(flat_data, flat_time, fs)
lr = pg.LinearRegionItem([0,20])
lr.setZValue(-10)
plt1.addItem(lr)
plt1.addItem(curve1)

vid = win.addViewBox(lockAspect = True, row = 1, col = 4)
img = pg.ImageItem(np.random.normal(size=(150,150)))
vid.addItem(img)
vid.autoRange()

win.nextRow()

plt = win.addPlot(row=2, col=1 , colspan = 4, title = 'tid ... ')
plt.enableAutoRange(False, False)
plt.setXRange(0, 500)
bx = plt.getViewBox()
curve = HDF5Plot(parent = plt, viewbox = bx)
curve.setHDF5(flat_data, flat_time, fs)
plt.addItem(curve)

def updatePlot():
    plt.setXRange(*lr.getRegion(), padding=0)
def updateRegion():
    lr.setRegion(plt.getViewBox().viewRange()[0])
lr.sigRegionChanged.connect(updatePlot)
plt.sigXRangeChanged.connect(updateRegion)
updatePlot()



def scroll():
    data1[:-1] = data1[1:]  # shift data in the array one sample left
                            # (see also: np.roll)
    data1[-1] = np.random.normal()
    curve1.setData(data1)

    ptr1 += 1
    curve2.setData(data1)
    curve2.setPos(ptr1, 0)


## Start Qt event loop unless running in interactive mode or using pyside.
if __name__ == '__main__':
    import sys
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        QtGui.QApplication.instance().exec_()


'''
from .ndf import H5File
from .ndf import NdfFile
from .ndf import plot
from .ndf import basic_plot
from .ndf import DataHandler
from .ndf import FeaturePreProcesser
from .ndf import Classifier
from .ndf import FeatureExtractor
from .ndf import load_classifier
from .ndf import make_hmm_model
from .visualisation import pyecog_main_gui

'''

from .ndf import *
from .visualisation import *




from .ndfconverter import NdfFile
from .h5loader import H5File
from .datahandler import DataHandler
from .classifier import FeaturePreProcesser
from .classifier import Classifier
from .classifier import load_classifier
#from .bokeh_visualisation import plot
#from .bokeh_visualisation import basic_plot
from .feature_extractor import FeatureExtractor
from .hmm import make_hmm_model

import os
import logging
#try:
#    logging.info('Re-intted')
#except:
logger = logging.getLogger()
logpath = os.getcwd()
#fhandler = logging.FileHandler(filename=os.path.join(os.path.split(logpath)[0], 'Datahandler.log'), mode='w')
fhandler = logging.FileHandler(filename=os.path.join(logpath, 'PyECoG_logfile.log'), mode='a+')
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
fhandler.setFormatter(formatter)
logger.addHandler(fhandler)
logger.setLevel(logging.DEBUG)


from __future__ import print_function
import time
import sys

import pickle
import numpy as np
import scipy.stats as st
#import matplotlib.pyplot as plt


from numpy import NaN, Inf, arange, isscalar, asarray, array # imports for the peakdet method


class FeatureExtractor():

    def __init__(self, dataset, subtract_baseline = True):
        self.start = time.clock()
        self.dataset = dataset
        self._baseline(subtract_baseline=subtract_baseline, threshold=0.04, window_size=100)
        self._event_dataset_stats()
        self._peaks_and_valleys()
        self._wavelet_features(fs = 512, frequencies = [1,5,10,15,20,30,60, 90])

        print("Stacking up...")
        self.feature_array = np.hstack([self.event_stats,self.baseline_features,
                                   self.pkval_stats,self.meanpower])

        print(str(self.feature_array.shape))
        print('took '+ str(time.clock()-self.start)+ ' seconds to extract '+ str(dataset.shape[0])+ 'feature vectors')


    def _event_dataset_stats(self):
        print("Extracting stats on event dataset...")
        self.event_stats = np.zeros((len(self.event_dataset),7))
        self.event_stats_col_labels = ('min','max','mean','std-dev','skew','kurtosis','sum(abs(difference))')

        for i,trace in enumerate(self.event_dataset):
            if trace.shape[0]:
                # if not all baseline
                self.event_stats[i,0] = np.min(trace)
                self.event_stats[i,1] = np.max(trace)
                self.event_stats[i,2] = np.mean(trace)
                self.event_stats[i,3] = np.std(trace)
                self.event_stats[i,4] = st.kurtosis(trace)
                self.event_stats[i,5] = st.skew(trace)
                self.event_stats[i,6] = np.sum(np.absolute(np.diff(trace)))
            else:
                 # if all baseline
                self.event_stats[i,:] = np.NaN

        print("DONE"+ str(time.clock() - self.start))

    def _baseline(self, subtract_baseline = True, threshold = 0.04, window_size = 100):
        '''
        Method calculates 2 feature lists and an event dataset:
            - self.event_dataset is the data window with 'baseline points removed'
            - baseline_length is the first feature list and is the number of datapoints within the window
              defined as being baseline using a rolling std deviation window and threshold
            - baseline_mean_diff is the second list and is the mean difference between baseline datapoint indexes.
        '''
        print('Extracting baseline via rolling std...')
        array_std = np.std(self.rolling_window(self.dataset,window=window_size),-1)
        array_window = np.zeros([self.dataset.shape[0],window_size-1])
        rolling_std_array = np.hstack((array_window,array_std))
        masked_std_below_threshold = np.ma.masked_where(rolling_std_array > threshold, self.dataset)
        mean_baseline_vector = np.mean(masked_std_below_threshold,axis = 1)

        if subtract_baseline:
            dataset_after_subtraction_option = self.dataset - mean_baseline_vector[:,None]
        else:
            dataset_after_subtraction_option = self.dataset

        masked_std_above_threshold = np.ma.masked_where(rolling_std_array < threshold, dataset_after_subtraction_option)
        indexes = np.array(np.arange(self.dataset.shape[1]))

        self.event_dataset = [np.ma.compressed(masked_std_above_threshold[i,:]) for i in xrange(self.dataset.shape[0])]
        baseline_length = [len(np.ma.compressed(masked_std_below_threshold[i,:])) for i in xrange(self.dataset.shape[0])]
        baseline_diff_skew = [st.skew(np.diff(indexes[np.logical_not(masked_std_below_threshold[i].mask)])) for i in xrange(self.dataset.shape[0])]
        baseline_mean_diff = [np.mean(np.diff(indexes[np.logical_not(masked_std_below_threshold[i].mask)])) for i in xrange(self.dataset.shape[0])]
        #self.baseline_diff = [indexes[np.logical_not(masked_std_below_threshold[i].mask)] for i in xrange(dataset.shape[0])]

        self.baseline_features = np.vstack([baseline_length,baseline_mean_diff,baseline_diff_skew]).T
        print('DONE'+ str(time.clock() - self.start))
        #print 'N.B using mean baseline difference - perhaps not best?'

    def _peaks_and_valleys(self):
        print ("Extracting peak/valley features..."),
        pk_nlist = []
        val_nlist = []

        for i in range(self.dataset.shape[0]):
            pk, val = self.peakdet(self.dataset[i,:], 0.5)
            pk_nlist.append(pk)
            val_nlist.append(val)

        # always 2 columns
        n_pks = []
        n_vals = []
        av_pk = []
        av_val = []
        av_range = []
        for i in range(len(pk_nlist)):
            n_pks.append(pk_nlist[i].shape[0])
            n_vals.append(val_nlist[i].shape[0])

            if pk_nlist[i].shape[0]:
                av_pk.append(np.mean(pk_nlist[i][:,1]))
            else:
                av_pk.append(np.NaN)

            if val_nlist[i].shape[0]:
                av_val.append(np.mean(val_nlist[i][:,1]))
            else:
                av_val.append(np.NaN)

            if val_nlist[i].shape[0] and pk_nlist[i].shape[0]:
                av_range.append(
                    abs(np.mean(pk_nlist[i][:,1])+np.mean(val_nlist[i][:,1]))
                    )
            else:
                av_range.append(np.NaN)

        n_pks = np.array(n_pks)
        n_vals = np.array(n_vals)
        av_pk = np.array(av_pk)
        av_val = np.array(av_val)
        av_range = np.array(av_range)
        self.pkval_stats = np.vstack([n_pks,n_vals,av_pk,av_val,av_range]).T
        print("DONE", time.clock()-self.start)
        #print ('N.B Again - preassign?')

    def _wavelet_features(self, fs = 512, frequencies = [1,5,10,15,20,30,60, 90]):
        print("Extracting wavelet features from event dataset..."),
        window = int(fs*0.5)
        waveletList = []
        for freq in frequencies:
            wavelet, xaxis = self.complexMorlet(freq,fs,timewindow = (-0.1,0.1))
            waveletList.append(wavelet)

        power = []
        meanpower = []
        for i in range(len(self.event_dataset)):
            if self.event_dataset[i].shape[0]:
                convResults = self.convolve(self.event_dataset[i], waveletList)
                powerArray  = np.absolute(convResults)
                #print powerArray.shape
                power.append(powerArray)
                meanpower.append(np.mean(powerArray,axis = 1))

            else:
                power.append(np.ones((len(frequencies)))*np.NaN)
                meanpower.append(np.ones((len(frequencies)))*np.NaN)
        self.meanpower = np.array(meanpower)
        print("DONE", time.clock()-self.start)

        #print("N.B. Still no post crossing power")

    @ staticmethod
    def rolling_window(array, window):
        """
        Remember that the rolling window actually starts at the window length in.
        """
        shape = array.shape[:-1] + (array.shape[-1] - window + 1, window)
        strides = array.strides + (array.strides[-1],)
        return np.lib.stride_tricks.as_strided(array, shape=shape, strides=strides)

    @staticmethod
    def peakdet(v, delta, x=None):
        """
        function [maxtab, mintab]=peakdet(v, delta, x)
                [MAXTAB, MINTAB] = PEAKDET(V, DELTA) finds the local
                maxima and minima ("peaks") in the vector V.
                MAXTAB and MINTAB consists of two columns. Column 1
                contains indices in V, and column 2 the found values.

                With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
                in MAXTAB and MINTAB are replaced with the corresponding
               X-values.

                A point is considered a maximum peak if it has the maximal
                value, and was preceded (to the left) by a value lower by
                DELTA.
        """
        maxtab = []
        mintab = []
        if x is None:
            x = arange(len(v))
        v = asarray(v)
        if len(v) != len(x):
            sys.exit('Input vectors v and x must have same length')
        if not isscalar(delta):
            sys.exit('Input argument delta must be a scalar')
        if delta <= 0:
            sys.exit('Input argument delta must be positive')
        mn, mx = Inf, -Inf
        mnpos, mxpos = NaN, NaN
        lookformax = True
        for i in arange(len(v)):
            this = v[i]
            if this > mx:
                mx = this
                mxpos = x[i]
            if this < mn:
                mn = this
                mnpos = x[i]
            if lookformax:
                if this < mx-delta:
                    maxtab.append((mxpos, mx))
                    mn = this
                    mnpos = x[i]
                    lookformax = False
            else:
                if this > mn+delta:
                    mintab.append((mnpos, mn))
                    mx = this
                    mxpos = x[i]
                    lookformax = True
        return array(maxtab), array(mintab)

    @staticmethod
    def complexMorlet(freq, samplingFreq, timewindow,
                  noWaveletCycles=6,freqNorm = False):
        '''
        freq              = wavelet frequency
        samplingFrequency = 'insert data sampling frequency here'
        noWaveletCycles   = 'reread variable name' - trades off temp and freq precision
        timewindow        = timewindow over which to calculate wavelet
        freqNorm          = Bool, decide whether to normlise over frequencies
        plotWavelet       = Plot the construction?
        '''
        n = noWaveletCycles
        f = freq
        timeaxis = np.linspace(timewindow[0],timewindow[1],samplingFreq*(2.*timewindow[1]))
        sine_wave = np.exp(2*np.pi*1j*f*timeaxis)
        gstd = n/(2.*np.pi*f)
        gauss_win = np.exp(-timeaxis**2./(2.*gstd**2.))

        A = 1 # just 1 if not normlising freqs
        if freqNorm:
            #gstd    = (4/(2*np.pi*f))**2# this is his?
            A = 1.0/np.sqrt((gstd*np.sqrt(np.pi)))
            print('freqNorm is :'+str(A)+' for'+str(f) +' Hz')

        wavelet = A*sine_wave*gauss_win
        #print wavelet.shape

        return wavelet, timeaxis

    @staticmethod
    def convolve(dataSection, wavelets):
        convResults = []
        for i in range(len(wavelets)):
            wavelet = wavelets[i]
            complexResult = np.convolve(dataSection,wavelet,'same')
            convResults.append(complexResult)

        convResults = np.array(convResults) # convert to a numpy array!
        #print convResults.shape
        return convResults




'''
This file was taken from internet, massive hammer used to just check the sampling frequency of abf files.

Makes a fs_dictionary to be used by network predictor.

'''


#@mfunction("d, si, h")
import os
import numpy as np
import struct

class Abf():
    def __init__(self, fn=None):
        self.fn = fn


    def abfload(self):
        # ** function [d,si,h]=abfload(fn,varargin)
        # loads and returns data in ABF (Axon Binary File) format.
        # Data may have been acquired in the following modes:
        # (1) event-driven variable-length (currently only abf versions < 2.0)
        # (2) event-driven fixed-length or waveform-fixed length
        # (3) gap-free
        # Information about scaling, the time base and the number of channels and
        # episodes is extracted from the header of the abf file.
        #
        # OPERATION
        # If the second input variable is the char array 'info' as in
        #         [d,si,h]=abfload('d:\data01.abf','info')
        # abfload will not load any data but return detailed information (header
        # parameters) on the file in output variable h. d and si will be empty.
        # In all other cases abfload will load data. Optional input parameters
        # listed below (= all except the file name) must be specified as
        # parameter/value pairs, e.g. as in
        #         d=abfload('d:\data01.abf','start',100,'stop','e'),
        #
        # >>> INPUT VARIABLES >>>
        # NAME        TYPE, DEFAULT      DESCRIPTION
        # fn          char array         abf data file name
        # start       scalar, 0          only gap-free-data: start of cutout to be
        #                                 read (unit: s)
        # stop        scalar or char,    only gap-free-data:  of cutout to be
        #             'e'                 read (unit: sec). May be set to 'e' (
        #                                 of file).
        # sweeps      1d-array or char,  only episodic data: sweep numbers to be
        #             'a'                 read. By default, all sweeps will be read
        #                                 ('a').
        # channels    cell array         names of channels to be read, like
        #              or char, 'a'       {'IN 0','IN 8'} (make sure spelling is
        #                                 100% correct, including blanks). If set
        #                                 to 'a', all channels will be read.
        # chunk       scalar, 0.05       only gap-free-data: the elementary chunk
        #                                 size (megabytes) to be used for the
        #                                 'discontinuous' mode of reading data
        #                                 (fewer channels to be read than exist)
        # machineF    char array,        the 'machineformat' input parameter of the
        #              'ieee-le'          matlab fopen function. 'ieee-le' is the
        #                                 correct option for windows, deping on
        #                                 the platform the data were recorded/shall
        #                                 be read by abfload 'ieee-be' is the
        #                                 alternative.
        # << OUTPUT VARIABLES <<<
        # NAME  TYPE            DESCRIPTION
        # d                     the data read, the format deping on the record-
        #                        ing mode
        #   1. GAP-FREE:
        #   2d array        2d array of size
        #                    <data pts> by <number of chans>
        #                    Examples of access:
        #                    d(:,2)       data from channel 2 at full length
        #                    d(1:100,:)   first 100 data points from all channels
        #   2. EPISODIC FIXED-LENGTH/WAVEFORM FIXED-LENGTH:
        #   3d array        3d array of size
        #                    <data pts per sweep> by <number of chans> by <number
        #                    of sweeps>.
        #                    Examples of access:
        #                    d(:,2,:)            a matrix containing all episodes
        #                                        (at full length) of the second
        #                                        channel in its columns
        #                    d(1:200,:,[1 11])   contains first 200 data points of
        #                                        episodes 1 and 11 of all channels
        #   3. EPISODIC VARIABLE-LENGTH:
        #   cell array      cell array whose elements correspond to single sweeps.
        #                    Each element is a (regular) array of size
        #                    <data pts per sweep> by <number of chans>
        #                    Examples of access:
        #                    d{1}            a 2d-array which contains episode 1
        #                                    (all of it, all channels)
        #                    d{2}(1:100,2)   a 1d-array containing the first 100
        #                                    data points of channel 2 in episode 1
        # si    scalar           the sampling interval in us
        # h     struct           information on file (selected header parameters)
        #
        #
        # CONTRIBUTORS
        #   Original version by Harald Hentschke (harald.hentschke@uni-tuebingen.de)
        #   Exted to abf version 2.0 by Forrest Collman (fcollman@Princeton.edu)
        #   pvpmod.m by Ulrich Egert (egert@bccn.uni-freiburg.de)
        #   Date of this version: May 20, 2009
        start = 0.0
        stop = 'e'
        # episodic
        sweeps = 'a'
        # general
        channels = 'a'
        # the size of data chunks (see above) in Mb. 0.05 Mb is an empirical value
        # which works well for abf with 6(1,1)6 channels and recording durations of
        # 5-30 min
        chunk = 0.05
        machineF = '<' #'ieee-le'
        verbose = 1
        # if first and only optional input argument is string 'info' the user's
        # request is to obtain information on the file (header parameters), so set
        # flag accordingly
        doLoadData = True

        # some constants
        BLOCKSIZE = 512
        # output variables
        d = []
        si = []
        h = []


        # check existence of file
        if not os.path.exists(self.fn):
            print 'could not find file '
            return

        # -------------------------------------------------------------------------
        #                       PART 2a: determine abf version
        # -------------------------------------------------------------------------
        fid = open(self.fn, 'rb')#, machineF)

        # on the occasion, determine absolute file size
        fileSz = os.stat(self.fn)[7]

        # *** read value of parameter 'fFileSignature' [i.e. abf version) from header ***
        sz = 4
        fFileSignature = char2str(struct.unpack('c'*sz,fid.read(sz)))#, ''B'=>char'))

        # rewind
        fid.seek(0)
        # transpose
        #fFileSignature = fFileSignature.cT

        # -------------------------------------------------------------------------
        #    PART 2b: define file information ('header') parameters of interest
        # -------------------------------------------------------------------------
        # The list of header parameters created below (variable 'headPar') is
        # derived from the abf version 1.8 header section. It is by no means
        # exhaustive [i.e. there are many more parameters in abf files) but
        # sufficient for proper upload, scaling and arrangement of data acquired
        # under many conditons. Further below, these parameters will be made fields
        # of struct h. h, which is also an output variable, is then used in PART 3,
        # which does the actual job of uploading, scaling and rearranging the data.
        # That part of the code relies on h having a certain set of fields
        # irrespective of ABF version.
        # Unfortunately, in the transition to ABF version 2.0 many of the header
        # parameters were moved to different places within the abf file and/or
        # given other names or completely restructured. In order for the code to
        # work with pre- and post-2.0 data files, all parameters missing in the
        # header must be gotten into h. This is accomplished in lines ~262 and
        # following:
        #     if h.fFileVersionNumber>=2
        #       ...
        # Furthermore,
        # - h as an output from an ABF version < 2.0 file will not contain new
        #   parameters introduced into the header like 'nCRCEnable'
        # - h will in any case contain a few 'home-made' fields that have
        #   proven to be useful. Some of them dep on the recording mode. Among
        #   the more or less self-explanatory ones are
        # -- si                   sampling interval
        # -- recChNames           the names of all channels, e.g. 'IN 8',...
        # -- dataPtsPerChan       sample points per channel
        # -- dataPts              sample points in file
        # -- recTime              recording start and stop time in seconds from
        #                         midnight (millisecond resolution)
        # -- sweepLengthInPts     sample points per sweep (one channel)
        # -- sweepStartInPts      the start times of sweeps in sample points
        #                         (from beginning of recording)


        # call local function for header proper
        headPar = self.define_header(fFileSignature)
        TagInfo = self.constTagInfo()
        if 'ABF' in fFileSignature:    # ** note the blank
            # ************************
            #     abf version < 2.0
            # ************************
            # do nothing, header already defined above
            pass
        elif 'ABF2' in fFileSignature:
            # ************************
            #     abf version >= 2.0
            # ************************
            Sections = self.Sections()
            ProtocolInfo = self.constProtocolInfo()
            ADCInfo = self.constADCInfo()
        else:
            print 'unknown or incompatible file signature'


        # convert headPar to struct
        #s = cell2struct(headPar, ('name'), 'offs'), 'numType'), 'value')), 2)

        # -------------------------------------------------------------------------
        #    PART 2c: read parameters of interest
        # -------------------------------------------------------------------------
        # convert names in structure to variables and read value from header
        h = {}
        for k,v in headPar.items():
            fid.seek(v[0])
               # fid.close()
               # print 'something went wrong locating '+ g).name]))
            #h[k] = struct.unpack(k[1], fid.read(sz)) #read value(s) into dictionary
            if hasattr(v[2],'size'):
                c = v[2].size
            else:
                c=1
            h[k] = np.fromfile(file=fid, dtype=v[1], count = v[2][0]*v[2][1] ).reshape(v[2])
            h[k] = h[k][0] # hack, how to get directly not a nested list?

        # transpose
        #h.fFileSignature = h.fFileSignature.cT
        # several header parameters need a fix or version-specific refinement:
        if 'ABF2' in fFileSignature:
            # h["fFileVersionNumber"] needs to be converted from an array of integers to
            # a 'f'
            h["fFileVersionNumber"] = h["fFileVersionNumber"][3] + h["fFileVersionNumber"][2] * .1 + h["fFileVersionNumber"][1] * .001 + h["fFileVersionNumber"][0] * .0001
            # convert ms to s
            h["lFileStartTime"] = h["uFileStartTimeMS"] * .001
        else:
            # h["fFileVersionNumber"] is a float32 the value of which is sometimes a
            # little less than what it should be (e.g. 1.6499999 instead of 1.65)
            h["fFileVersionNumber"] = .001 * round(h["fFileVersionNumber"] * 1000)
            # in abf < 2.0 two parameters are needed to obtain the file start time
            # with millisecond precision - let's integrate both into parameter
            # lFileStartTime (unit: s) so that nFileStartMillisecs will not be needed
            h["lFileStartTime"] = h["lFileStartTime"] + h["nFileStartMillisecs"] * .001


        # *** read file information that has gone elsewhere in ABF version >= 2.0
        # and assign values ***
        if h["fFileVersionNumber"] >= 2:
            # --- read in the Sections
            offset = 76
            for k,v in Sections.items():
                exec(k+'='+ ReadSectionInfo(fid,offset)) in locals()
                offset = offset + 4 + 4 + 8

            # --- read in the Strings
            fid.seek(StringsSection["uBlockIndex"] * BLOCKSIZE)
            BigString = struct.unpack('c',fid.read(StringsSection["uBytes"]))#, 'char'))
            # this is a hack
            goodstart = lower(BigString).find('clampex')
            #this exts the hack to deal with axoscope files
            if goodstart > -1:
                goodstart = lower(BigString).find('axoscope')


            BigString = BigString[goodstart[0]:]
            stringends = (BigString == 0).nonzero()
            stringends = [0].append(stringends)
            Strings = []
            for i in range(len(stringends) - 1):
                Strings[i] = str(BigString[stringends[i]+1:stringends[i+1]-1])

            h["recChNames"] = []
            h["recChUnits"] = []

            # --- read in the ADCSection
            for i in range(ADCSection["llNumEntries"]):
                ADCsec[i] = ReadSection(fid, ADCSection["uBlockIndex"] * BLOCKSIZE + ADCSection["uBytes"] * (i - 1), ADCInfo)
                ii = ADCsec[i].nADCNum + 1
                h["nADCSamplingSeq"][i] = ADCsec[i].nADCNum
                h["recChNames"] = strvcat(h["recChNames"], Strings(ADCsec[i].lADCChannelNameIndex))
                h["recChUnits"] = strvcat(h["recChUnits"], Strings(ADCsec[i].lADCUnitsIndex))
                h["nTelegraphEnable"][ii] = ADCsec[i].nTelegraphEnable
                h["fTelegraphAdditGain"][ii] = ADCsec[i].fTelegraphAdditGain
                h["fInstrumentScaleFactor"][ii] = ADCsec[i].fInstrumentScaleFactor
                h["fSignalGain"][ii] = ADCsec[i].fSignalGain
                h["fADCProgrammableGain"][ii] = ADCsec[i].fADCProgrammableGain
                h["fInstrumentOffset"][ii] = ADCsec[i].fInstrumentOffset
                h["fSignalOffset"][ii] = ADCsec[i].fSignalOffset

            # --- read in the protocol section
            ProtocolSec = ReadSection(fid, ProtocolSection["uBlockIndex"] * BLOCKSIZE, ProtocolInfo)
            # ---
            h["nOperationMode"] = ProtocolSec["nOperationMode"]
            h["fSynchTimeUnit"] = ProtocolSec["fSynchTimeUnit"]
            h["nADCNumChannels"] = ADCSection["llNumEntries"]
            h["lActualAcqLength"] = DataSection["llNumEntries"]
            h["lDataSectionPtr"] = DataSection["uBlockIndex"]
            h["nNumPointsIgnored"] = 0
            # in ABF version < 2.0 h["fADCSampleInterval"] is the sampling interval
            # defined as
            #     1/(sampling freq*number_of_channels)
            # so divide ProtocolSec.fADCSequenceInterval by the number of
            # channels
            h["fADCSampleInterval"] = ProtocolSec["fADCSequenceInterval"] / h["nADCNumChannels"]
            h["fADCRange"] = ProtocolSec["fADCRange"]
            h["lADCResolution"] = ProtocolSec["lADCResolution"]
            h["lSynchArrayPtr"] = SynchArraySection["uBlockIndex"]
            h["lSynchArraySize"] = SynchArraySection["llNumEntries"]
        else:
            TagSection = {}
            TagSection["llNumEntries"] = h["lNumTagEntries"]
            TagSection["uBlockIndex"] = h["lTagSectionPtr"]
            TagSection["uBytes"] = 64

        Tagsec = []
        for i in range(TagSection["llNumEntries"]):
            Tagsec[i] = ReadSection(fid, TagSection["uBlockIndex"] * BLOCKSIZE + TagSection["uBytes"] * (i - 1), TagInfo)
            Tagsec[i]["sComment"] = Tagsec[i]["sComment"]

        h["Tags"] = Tagsec


        # -------------------------------------------------------------------------
        #    PART 2d: groom parameters & perform some plausibility checks
        # -------------------------------------------------------------------------
        if h["lActualAcqLength"] < h["nADCNumChannels"]:
            fid.close()
            print 'less data points than sampled channels in file'

        # the numerical value of all recorded channels (numbers 0..15)
        recChIdx = h["nADCSamplingSeq"][0:h["nADCNumChannels"]]
        # the corresponding indices into loaded data d
        recChInd = range(len(recChIdx))
        if h["fFileVersionNumber"] < 2:
            # the channel names, e.g. 'IN 8' (for ABF version 2.0 these have been
            # extracted above at this point)
            qq = h["sADCChannelName"].reshape((16, 10))
            del h["sADCChannelName"]
            qq = qq[recChIdx,:]
            h["recChNames"] = []
            for rc in range(qq.shape[0]):
                h["recChNames"].append(struct.unpack(str(qq.shape[1])+'s',qq[rc,:])[0])
            # same with signal units
            qq = h["sADCUnits"].reshape((16,8))
            del h["sADCUnits"]
            qq = qq[recChIdx,:]
            h["recChUnits"] = []
            for rc in range(qq.shape[0]):
                h["recChUnits"].append(struct.unpack(str(qq.shape[1])+'s',qq[rc,:])[0])


        # convert to cell arrays
     #   h["recChNames"] = deblank(cellstr(h["recChNames"]))
      #  h["recChUnits"] = deblank(cellstr(h["recChUnits"]))

        # check whether requested channels exist
        chInd = []
        eflag = 0
        if isinstance(channels, basestring):
            if channels == 'a':
                chInd = recChInd
            else:
                fid.close()
                print 'input parameter \'channels\' must either be a cell array holding channel names or the single character \'a\' (=all channels)'

        else:
            for i in range(len(channels)):
                tmpChInd = h["recChNames"].find(channels[i])
                if len(tmpChInd) > 0:
                    chInd = [chInd, tmpChInd]
                else:
                    # set error flag to 1
                    eflag = 1
        if eflag:
            fid.close()
            print'**** available channels:'
            print h["recChNames"]
            print' '
            print'**** requested channels:'
            print channels
            print 'at least one of the requested channels does not exist in data file (see above)'

        # display available channels if in info mode
        if not doLoadData:
            print '**** available channels:'
            print h["recChNames"]


        # gain of telegraphed instruments, if any
        if h["fFileVersionNumber"] >= 1.65:
            addGain = h["nTelegraphEnable"] * h["fTelegraphAdditGain"]
            addGain = np.where(addGain==0, 1, addGain)#addGain(addGain == 0] = 1
        else:
            addGain = np.ones(h["fTelegraphAdditGain"].shape)


        # determine offset at which data start
        if h["nDataFormat"] == 0:
            dataSz = 2    # bytes/point
            precision = 'i2' #int16
        elif h["nDataFormat"] == 1:
            dataSz = 4    # bytes/point
            precision = 'f'
        else:
            fid.close()
            print 'invalid number format'

        headOffset = h["lDataSectionPtr"] * BLOCKSIZE + h["nNumPointsIgnored"] * dataSz
        # h["fADCSampleInterval"] is the TOTAL sampling interval
        h["si"] = h["fADCSampleInterval"] * h["nADCNumChannels"]
        # assign same value to si, which is an output variable
        si = h["si"]
        if isinstance(sweeps,basestring) and sweeps == 'a':
            nSweeps = h["lActualEpisodes"]
            sweeps = np.arange(h["lActualEpisodes"])
        else:
            nSweeps = len(sweeps)


        # -------------------------------------------------------------------------
        #    PART 3: read data (note: from here on code is generic and abf version
        #    should not matter)
        # -------------------------------------------------------------------------
        if h["nOperationMode"] == 1:
            print 'data were acquired in event-driven variable-length mode'
            if h["fFileVersionNumber"] >= 2.0:
                print 'abfload currently does not work with data acquired in event-driven variable-length mode and ABF version 2.0'
            else:
                if (h["lSynchArrayPtr"] <= 0 or h["lSynchArraySize"] <= 0):
                    fid.close()
                    print 'internal variables \'lSynchArraynnn\' are zero or negative'

                if h["fSynchTimeUnit"] == 0:            # time information in synch array section is in terms of ticks
                    h["synchArrTimeBase"] = 1
                else:            # time information in synch array section is in terms of usec
                    h["synchArrTimeBase"] = h["fSynchTimeUnit"]

                # the byte offset at which the SynchArraySection starts
                h["lSynchArrayPtrByte"] = BLOCKSIZE * h["lSynchArrayPtr"]
                # before reading Synch Arr parameters check if file is big enough to hold them
                # 4 bytes/long, 2 values per episode (start and length)
                if h["lSynchArrayPtrByte"] + 2 * 4 * h["lSynchArraySize"] < fileSz:
                    fid.close()
                    print 'file seems not to contain complete Synch Array Section'

                synchArr = struct.unpack('i', fid.read(h["lSynchArraySize"] * 2))
                if h["lSynchArraySize"] * 2 != len(synchArr):
                    fid.close()
                    print 'something went wrong reading synch array section'

                # make synchArr a h["lSynchArraySize"] x 2 matrix
                synchArr = synchArr.cT.reshape(2, h["lSynchArraySize"])[2:1]
                # the length of episodes in sample points
                segLengthInPts = synchArr[:, 2] / h["synchArrTimeBase"]
                # the starting ticks of episodes in sample points WITHIN THE DATA FILE
                segStartInPts = np.array([0, segLengthInPts[:-2].cT] * dataSz).cumsum() + headOffset
                # start time (synchArr(:,1)) has to be divided by h["nADCNumChannels"] to get true value
                # go to data portion

                # ** load data if requested
                if doLoadData:
                    for i in range(nSweeps):
                        # if selected sweeps are to be read, seek correct position
                        if nSweeps != h["lActualEpisodes"]:
                            fid.seek(segStartInPts[sweeps[i]])

                        tmpd = np.fromfile(file=fid, dtype=precision, count = segLengthInPts[sweeps[i]])
                        #tmpd = struct.unpack(precision, fid.read(segLengthInPts[sweeps[i]]))
              #          if n != segLengthInPts(sweeps(i]):
               #             print 'something went wrong reading episode '+str(sweeps(i])+ ': '+ segLengthInPts(sweeps(i])+' points should have been read, ' +str(n)+ ' points actually read'

                        h["dataPtsPerChan"] = tmpd.size / h["nADCNumChannels"]
                        if n % h["nADCNumChannels"] > 0:
                            fid.close()
                            print 'number of data points in episode not OK'
                        # separate channels..
                        tmpd = tmpd.reshape((h["nADCNumChannels"], h["dataPtsPerChan"]))
                        # retain only requested channels
                        tmpd = tmpd[chInd,:].cT
                        # if data format is integer, scale appropriately, if it's 'f', tmpd is fine
                        if not h["nDataFormat"]:
                            for j in range(len(chInd)):
                                ch = recChIdx[chInd[j]] + 1
                                tmpd[:,j] = tmpd[:, j] / (h["fInstrumentScaleFactor"][ch] \
                                              * h["fSignalGain"][ch] * h["fADCProgrammableGain"][ch] \
                                              * addGain[ch]) * h["fADCRange"] / \
                                              h["lADCResolution"] + h["fInstrumentOffset"][ch] - h["fSignalOffset"][ch]
                        # now place in cell array, an element consisting of one sweep with channels in columns
                        d[i] = tmpd
        elif h["nOperationMode"]  in (2, 5):
            if h["nOperationMode"] == 2:
                print'data were acquired in event-driven fixed-length mode'
            else:
                print'data were acquired in waveform fixed-length mode (clampex only)'

            # extract timing information on sweeps
            if (h["lSynchArrayPtr"] <= 0 or h["lSynchArraySize"] <= 0):
                fid.close()
                print 'internal variables \'lSynchArraynnn\' are zero or negative'

            if h["fSynchTimeUnit"] == 0:        # time information in synch array section is in terms of ticks
                h["synchArrTimeBase"] = 1
            else:        # time information in synch array section is in terms of usec
                h["synchArrTimeBase"] = h["fSynchTimeUnit"]

            # the byte offset at which the SynchArraySection starts
            h["lSynchArrayPtrByte"] = BLOCKSIZE * h["lSynchArrayPtr"]
            # before reading Synch Arr parameters check if file is big enough to hold them
            # 4 bytes/long, 2 values per episode (start and length)
            if h["lSynchArrayPtrByte"] + 2 * 4 * h["lSynchArraySize"] > fileSz:
                fid.close()
                print 'file seems not to contain complete Synch Array Section'

            fid.seek(h["lSynchArrayPtrByte"])
            #    fid.close()
             #   print 'something went wrong positioning file pointer to Synch Array Section'

            synchArr = np.fromfile(file=fid, dtype='i', count = h["lSynchArraySize"] * 2)
            #struct.unpack('i',fid.read(h["lSynchArraySize"] * 2))
            #if n != h["lSynchArraySize"] * 2:
             #   fid.close()
              #  print 'something went wrong reading synch array section'

            # make synchArr a h["lSynchArraySize"] x 2 matrix
           # synchArr = synchArr.reshape((2,h["lSynchArraySize"]))[2, 1] # rearrange dimension, why???
            if np.unique(synchArr.transpose()[1]).size > 1:
                fid.close()
                print 'sweeps of unequal length in file recorded in fixed-length mode'

            # the length of sweeps in sample points (**note: parameter lLength of
            # the ABF synch section is expressed in samples (ticks) whereas
            # parameter lStart is given in synchArrTimeBase units)
            h["sweepLengthInPts"] = np.unique(synchArr[1]) / h["nADCNumChannels"]
            # the starting ticks of episodes in sample points (t0=1=beginning of
            # recording)
            h["sweepStartInPts"] = np.unique(synchArr[0]) * (h["synchArrTimeBase"] / h["fADCSampleInterval"] / h["nADCNumChannels"])
            # recording start and stop times in seconds from midnight
            h["recTime"] = h["lFileStartTime"]
            h["recTime"] = h["recTime"] + np.hstack(([0], 1e-6 * (h["sweepStartInPts"] + h["sweepLengthInPts"]) * h["fADCSampleInterval"] * h["nADCNumChannels"]))
            # determine first point and number of points to be read
            startPt = 0
            h["dataPts"] = h["lActualAcqLength"]
            h["dataPtsPerChan"] = h["dataPts"] / h["nADCNumChannels"]
            if h["dataPts"] % h["nADCNumChannels"] > 0 or h["dataPtsPerChan"] % h["lActualEpisodes"] > 0:      #remainders
                fid.close()
                print 'number of data points not OK'

            # temporary helper var
            dataPtsPerSweep = h["sweepLengthInPts"] * h["nADCNumChannels"]
            fid.seek(startPt * dataSz + headOffset) #!= 0:
            #    fid.close()
             #   print 'something went wrong positioning file pointer (too few data points ?)'

            d = np.zeros((h["sweepLengthInPts"], len(chInd), nSweeps),dtype = precision)
            # the starting ticks of episodes in sample points WITHIN THE DATA FILE
            selectedSegStartInPts = (sweeps * dataPtsPerSweep) * dataSz + headOffset
            # ** load data if requested
            if doLoadData:
                for i in range(nSweeps):
                    fid.seek(selectedSegStartInPts[i])
                    tmpd = np.fromfile(file=fid, dtype=precision, count = dataPtsPerSweep)
                    h["dataPtsPerChan"] = tmpd.size / h["nADCNumChannels"][0]  #one of the arguments is not list, address explicitly
                    if tmpd.size % h["nADCNumChannels"] > 0:
                        fid.close()
                        print 'number of data points in episode not OK'

                    # separate channels..
                    tmpd = tmpd.reshape((h["dataPtsPerChan"],h["nADCNumChannels"]))
                    # retain only requested channels
                    tmpd = tmpd[:,chInd]
                   # tmpd = tmpd.cT
                    # if data format is integer, scale appropriately, if it's 'f', d is fine
                    if not h["nDataFormat"]:
                        for j in range(len(chInd)):
                            ch = recChIdx[chInd[j]] # + 1
                            tmpd[:,j] = tmpd[:,j] / (h["fInstrumentScaleFactor"][ch] * h["fSignalGain"][ch] * h["fADCProgrammableGain"][ch] * addGain[ch]) * h["fADCRange"][0] / h["lADCResolution"][0] + h["fInstrumentOffset"][ch] - h["fSignalOffset"][ch]

                    # now fill 3d array
                    d[:,:, i] = tmpd

        elif h["nOperationMode"] == 3:
            #print'data were acquired in gap-free mode'
            # from start, stop, headOffset and h["fADCSampleInterval"] calculate first point to be read
            #  and - unless stop is given as 'e' - number of points
            startPt = np.floor(1e6 * start * (1 / h["fADCSampleInterval"]))
            # this corrects undesired shifts in the reading frame due to rounding errors in the previous calculation
            startPt = np.floor(startPt / h["nADCNumChannels"]) * h["nADCNumChannels"]
            # if stop is a char array, it can only be 'e' at this point (other values would have
            # been caught above)
            if isinstance(stop,basestring):
                h["dataPtsPerChan"] = h["lActualAcqLength"] / h["nADCNumChannels"] - np.floor(1e6 * start / h["si"])
                h["dataPts"] = h["dataPtsPerChan"] * h["nADCNumChannels"]
            else:
                h["dataPtsPerChan"] = np.floor(1e6 * (stop - start) * (1 / h["si"]))
                h["dataPts"] = h["dataPtsPerChan"] * h["nADCNumChannels"]
                if h["dataPts"] <= 0:
                    fid.close()
                    print 'start is larger than or equal to stop'


            if h["dataPts"] % h["nADCNumChannels"] > 0:
                fid.close()
                print 'number of data points not OK'

            tmp = 1e-6 * h["lActualAcqLength"] * h["fADCSampleInterval"]
 #           if verbose:
  #              print 'total length of recording: '), num2str(tmp, '%5.1f')), ' s ~ '), num2str(tmp / 60, '%3.0f')), ' min')
                # 8 bytes per data point expressed in Mb
   #             print 'memory requirement for complete upload in matlab: '), num2str(round(8 * h["lActualAcqLength"] / 2 ** 20)), ' MB')

            # recording start and stop times in seconds from midnight
            h["recTime"] = h["lFileStartTime"]
            h["recTime"] = np.hstack([h["recTime"], h["recTime"] + tmp])
            #if fid.seek(startPt * dataSz + headOffset) != 0:
            #    fid.close()
            #    print 'something went wrong positioning file pointer (too few data points ?)'

            if doLoadData:
                # *** decide on the most efficient way to read data:
                # [i] all (of one or several) channels requested: read, done
                # [ii] one (of several) channels requested: use the 'skip' feature of
                # fread
                # [iii] more than one but not all (of several) channels requested:
                # 'discontinuous' mode of reading data. Read a reasonable chunk of data
                # (all channels), separate channels, discard non-requested ones [if
                # any), place data in preallocated array, repeat until done. This is
                # faster than reading the data in one big lump, separating channels and
                # discarding the ones not requested
                if len(chInd) == 1 and h["nADCNumChannels"] > 1:
                    # --- situation [ii]
                    # jump to proper reading frame position in file
                    if fid.seek((chInd - 1) * dataSz) != 0:
                        fid.close()
                        print 'something went wrong positioning file pointer (too few data points ?)'
                    skip = dataSz * (h["nADCNumChannels"] - 1)
                    prec = precision + 'x'*skip #pad bytes
                    nbytes=struct.calcsize(prec)
                    # read, skipping h["nADCNumChannels"]-1 data points after each read
                    d = struct.unpack(prec, fid.read(h["dataPtsPerChan"]*nbytes))
                #    if n != h["dataPtsPerChan"]:
                 #       fid.close()
                  #      print 'something went wrong reading file ('+ str(h["dataPtsPerChan"])+ ' points should have been read, '+str(n)+ ' points actually read'

                elif len(chInd) / h["nADCNumChannels"] < 1:
                    # --- situation [iii]
                    # prepare chunkwise upload:
                    # preallocate d
                    d = np.empty((h["dataPtsPerChan"], len(chInd)))
                    # the number of data points corresponding to the maximal chunk size,
                    # rounded off such that from each channel the same number of points is
                    # read (do not forget that each data point will by default be made a
                    # double of 8 bytes, no matter what the original data format is)
                    chunkPtsPerChan = floor(chunk * 2 ** 20 / 8 / h["nADCNumChannels"])
                    chunkPts = chunkPtsPerChan * h["nADCNumChannels"]
                    # the number of those chunks..
                    nChunk = floor(h["dataPts"] / chunkPts)
                    # ..and the remainder
                    restPts = h["dataPts"] - nChunk * chunkPts
                    restPtsPerChan = restPts / h["nADCNumChannels"]
                    # chunkwise row indices into d
                    dix = range(1,chunkPtsPerChan,h["dataPtsPerChan"])
                    dix[:, 2] = dix[:, 1] + chunkPtsPerChan - 1
                    dix[-1, 2] = h["dataPtsPerChan"]
              #      if verbose and nChunk:
               #         printmcat(['reading file in '), int2str(nChunk), ' chunks of ~'), num2str(chunk), ' Mb')]))

                    # do it
                    for ci in mslice[1:size(dix, 1) - 1]:
                        tmpd = struct.unpack(precision, fid.read(chunkPts))
              #          if n != chunkPts:
               #             fid.close()
                #            print mcat(['something went wrong reading chunk #'), int2str(ci], ' ('), int2str(chunkPts), ' points should have been read, '), int2str(n), ' points actually read')]))

                        # separate channels..
                        tmpd = tmpd.reshape((h["nADCNumChannels"], chunkPtsPerChan))
                        d[dix[ci, 1]:dix[ci, 2], :] = tmpd[chInd, :].cT

                    # collect the rest, if any
                    if restPts:
                        tmpd = struct.unpack(precision, fid.read(restPts))
                       # if n != restPts:
                        #    fid.close()
                         #   print mcat(['something went wrong reading last chunk ('), int2str(restPts), ' points should have been read, '), int2str(n), ' points actually read')]))

                        # separate channels..
                        tmpd = tmpd.reshape((h["nADCNumChannels"], restPtsPerChan))
                        d[dix[-1, 1]:dix[-1, 2], :] = tmpd[chInd, :].cT

                else:
                    # --- situation [i]
                    #d = struct.unpack(precision, fid.read(h["dataPts"])
                    d = np.fromfile(file=fid, dtype=precision, count = h["dataPts"]).reshape((h["nADCNumChannels"], h["dataPtsPerChan"]))
                   # if n != h["dataPts"]:
                    #    fid.close()
                     #   print mcat(['something went wrong reading file ('), int2str(h["dataPts"]), ' points should have been read, '), int2str(n), ' points actually read')]))

                    # separate channels..
                  #  d = reshape(d,
                    #d = d'

                # if data format is integer, scale appropriately, if it's 'f', d is fine
                if not h["nDataFormat"]:
                    for j in range(len(chInd)):
                        ch = recChIdx[chInd[j]] + 1
                        scaling_factor = 1.0/ ((h["fInstrumentScaleFactor"][ch] * h["fSignalGain"][ch] * h["fADCProgrammableGain"][ch] * addGain[ch]) * h["fADCRange"] / h["lADCResolution"] + h["fInstrumentOffset"][ch] - h["fSignalOffset"][ch])
                        #print "scaling data with scaling factor of %f" % scaling_factor
                        #print d
                        tmp = np.empty(d.shape, dtype=np.float64)

                        tmp[:][j] = d[:][j] / (h["fInstrumentScaleFactor"][ch] * h["fSignalGain"][ch] * h["fADCProgrammableGain"][ch] * addGain[ch]) * h["fADCRange"] / h["lADCResolution"] + h["fInstrumentOffset"][ch] - h["fSignalOffset"][ch]

                        d = tmp
                        #print d
        else:
            print'recording mode is \'high-speed oscilloscope\' which is not implemented -- returning empty matrix'
            d = []
            h["si"] = []

        fid.close()
        return d, h

    def constSections(self):
        Sections =['ProtocolSection',
         'ADCSection',
         'DACSection',
         'EpochSection',
         'ADCPerDACSection',
         'EpochPerDACSection',
         'UserListSection',
         'StatsRegionSection',
         'MathSection',
         'StringsSection',
         'DataSection',
         'TagSection',
         'ScopeSection',
         'DeltaSection',
         'VoiceTagSection',
         'SynchArraySection',
         'AnnotationSection',
         'StatsSection']
        return Sections

    def define_header(self,fileSig=None):
        if 'ABF' in fileSig:    # ** note the blank
            # ************************
            #     abf version < 2.0
            # ************************
            #
            # temporary initializing var
            tmp = repmat(-1, 1, 16)
            # define vital header parameters and initialize them with -1: set up a cell
            # array (and convert it to a struct below, which is more convenient)
            # column order is
            #        name, position in header in bytes, type, value)
            headpar = {
                       'fFileSignature':(0,'c',(4,1)),
                        'fFileVersionNumber':(4,'f',(1,1)),
                        'nOperationMode':(8,'i2',(1,1)),
                        'lActualAcqLength':(10,'i',(1,1)),
                        'nNumPointsIgnored':(14,'i2',(1,1)),
                        'lActualEpisodes':(16,'i',(1,1)),
                        'lFileStartTime':(24,'i',(1,1)),
                        'lDataSectionPtr':(40,'i',(1,1)),
                        'lTagSectionPtr':(44,'i',(1,1)),
                        'lNumTagEntries':(48,'i',(1,1)),
                        'lSynchArrayPtr':(92,'i',(1,1)),
                        'lSynchArraySize':(96,'i',(1,1)),
                        'nDataFormat':(100,'i2',(1,1)),
                        'nADCNumChannels':(120,'i2',(1,1)),
                        'fADCSampleInterval':(122,'f',(1,1)),
                        'fSynchTimeUnit':(130,'f',(1,1)),
                        'lNumSamplesPerEpisode':(138,'i',(1,1)),
                        'lPreTriggerSamples':(142,'i',(1,1)),
                        'lEpisodesPerRun':(146,'i',(1,1)),
                        'fADCRange':(244,'f',(1,1)),
                        'lADCResolution':(252,'i',(1,1)),
                        'nFileStartMillisecs':(366,'i2',(1,1)),
                        'nADCPtoLChannelMap':(378,'i2',(1,16)),
                        'nADCSamplingSeq':(410,'i2',(1,16)),
                        'sADCChannelName':(442,'u1',(1, 160)),
                        'sADCUnits':(602,'u1',(1, 128)),
                        'fADCProgrammableGain':(730,'f',(1,16)),
                        'fInstrumentScaleFactor':(922,'f',(1,16)),
                        'fInstrumentOffset':(986,'f',(1,16)),
                        'fSignalGain':(1050,'f',(1,16)),
                        'fSignalOffset':(1114,'f',(1,16)),
                        'nTelegraphEnable':(4512,'i2',(1,16)),
                        'fTelegraphAdditGain':(4576,'f',(1,16)) }

        elif 'ABF2' in fileSig:
            # ************************
            #     abf version >= 2.0
            # ************************
            headpar = {
                       'fFileSignature':(1,'S',(1,4)),
                       'fFileVersionNumber':(4,'i',(1,4)), #bit8=>int ???
                       'uFileInfoSize':(8,'I',-(1,1)),
                       'lActualEpisodes':(12,'I',(1,1)),
                       'uFileStartDate':(16,'I',(1,1)),
                       'uFileStartTimeMS':(20,'I',(1,1)),
                       'uStopwatchTime':(24,'I',(1,1)),
                       'nFileType':(28,'s',(1,1)),
                        'nDataFormat':(30,'s',(1,1)),
                        'nSimultaneousScan':(32,'s',(1,1)),
                        'nCRCEnable':(34,'s',(1,1)),
                        'uFileCRC':(36,'I',(1,1)),
                        'FileGUID':(40,'I',(1,1)),
                        'uCreatorVersion':(56,'I',(1,1)),
                        'uCreatorNameIndex':(60,'I',(1,1)),
                        'uModifierVersion':(64,'I',(1,1)),
                        'uModifierNameIndex':(68,'I',(1,1)),
                        'uProtocolPathIndex':(72,'I',(1,1))}
        return headpar

    def constProtocolInfo(self):
        ProtocolInfo={
            'nOperationMode':('s',1),
            'fADCSequenceInterval':('f',1),
            'bEnableFileCompression':('?',1),
            'sUnused1':('c',3),
            'uFileCompressionRatio':('I',1),
            'fSynchTimeUnit':('f',1),
            'fSecondsPerRun':('f',1),
            'lNumSamplesPerEpisode':('i',1),
            'lPreTriggerSamples':('i',1),
            'lEpisodesPerRun':('i',1),
            'lRunsPerTrial':('i',1),
            'lNumberOfTrials':('i',1),
            'nAveragingMode':('s',1),
            'nUndoRunCount':('s',1),
            'nFirstEpisodeInRun':('s',1),
            'fTriggerThreshold':('f',1),
            'nTriggerSource':('s',1),
            'nTriggerAction':('s',1),
            'nTriggerPolarity':('s',1),
            'fScopeOutputInterval':('f',1),
            'fEpisodeStartToStart':('f',1),
            'fRunStartToStart':('f',1),
            'lAverageCount':('i',1),
            'fTrialStartToStart':('f',1),
            'nAutoTriggerStrategy':('s',1),
            'fFirstRunDelayS':('f',1),
            'nChannelStatsStrategy':('s',1),
            'lSamplesPerTrace':('i',1),
            'lStartDisplayNum':('i',1),
            'lFinishDisplayNum':('i',1),
            'nShowPNRawData':('s',1),
            'fStatisticsPeriod':('f',1),
            'lStatisticsMeasurements':('i',1),
            'nStatisticsSaveStrategy':('s',1),
            'fADCRange':('f',1),
            'fDACRange':('f',1),
            'lADCResolution':('i',1),
            'lDACResolution':('i',1),
            'nExperimentType':('s',1),
            'nManualInfoStrategy':('s',1),
            'nCommentsEnable':('s',1),
            'lFileCommentIndex':('i',1),
            'nAutoAnalyseEnable':('s',1),
            'nSignalType':('s',1),
            'nDigitalEnable':('s',1),
            'nActiveDACChannel':('s',1),
            'nDigitalHolding':('s',1),
            'nDigitalInterEpisode':('s',1),
            'nDigitalDACChannel':('s',1),
            'nDigitalTrainActiveLogic':('s',1),
            'nStatsEnable':('s',1),
            'nStatisticsClearStrategy':('s',1),
            'nLevelHysteresis':('s',1),
            'lTimeHysteresis':('i',1),
            'nAllowExternalTags':('s',1),
            'nAverageAlgorithm':('s',1),
            'fAverageWeighting':('f',1),
            'nUndoPromptStrategy':('s',1),
            'nTrialTriggerSource':('s',1),
            'nStatisticsDisplayStrategy':('s',1),
            'nExternalTagType':('s',1),
            'nScopeTriggerOut':('s',1),
            'nLTPType':('s',1),
            'nAlternateDACOutputState':('s',1),
            'nAlternateDigitalOutputState':('s',1),
            'fCellID':('f',3),
            'nDigitizerADCs':('s',1),
            'nDigitizerDACs':('s',1),
            'nDigitizerTotalDigitalOuts':('s',1),
            'nDigitizerSynchDigitalOuts':('s',1),
            'nDigitizerType':('s',1),
         }
        return ProtocolInfo

    def constADCInfo(self):
        ADCInfo={
            'nADCNum':('s',1),
            'nTelegraphEnable':('s',1),
            'nTelegraphInstrument':('s',1),
            'fTelegraphAdditGain':('f',1),
            'fTelegraphFilter':('f',1),
            'fTelegraphMembraneCap':('f',1),
            'nTelegraphMode':('s',1),
            'fTelegraphAccessResistance':('f',1),
            'nADCPtoLChannelMap':('s',1),
            'nADCSamplingSeq':('s',1),
            'fADCProgrammableGain':('f',1),
            'fADCDisplayAmplification':('f',1),
            'fADCDisplayOffset':('f',1),
            'fInstrumentScaleFactor':('f',1),
            'fInstrumentOffset':('f',1),
            'fSignalGain':('f',1),
            'fSignalOffset':('f',1),
            'fSignalLowpassFilter':('f',1),
            'fSignalHighpassFilter':('f',1),
            'nLowpassFilterType':('c',1),
            'nHighpassFilterType':('c',1),
            'fPostProcessLowpassFilter':('f',1),
            'nPostProcessLowpassFilterType':('c',1),
            'bEnabledDuringPN':('?',1),
            'nStatsChannelPolarity':('s',1),
            'lADCChannelNameIndex':('i',1),
            'lADCUnitsIndex':('i',1),
         }
        return ADCInfo

    def constTagInfo(self):
        TagInfo={
            'lTagTime':('i',1),
            'sComment':('c',56),
            'nTagType':('s',1),
            'nVoiceTagNumber_or_AnnotationIndex':('s',1)
        }
        return TagInfo

def ReadSection(fid,offset,Format):
#s=cell2struct(Format,{'name','numType','number'},2),
    fid.seek(offset)
    Section ={}
    for k,v in Format.items(): # i=1:length(s)
        Section[k] =struct.unpack(v[1],fid.read(len(Format)))
    return Section

def ReadSectionInfo(fid,offset):
    fid.seek(offset)
    SectionInfo = {}
    SectionInfo["uBlockIndex"]=struct.unpack('I',fid.read(4))
    fid.seek(offset+4)
    SectionInfo["uBytes"]=struct.unpack("I",fid.read(4))#,''I''),
    fid.seek(offset+8)
    SectionInfo["llNumEntries"]=struct.unpack("f",fid.read(8))
    return SectionInfo

def repmat(a,m,n):
    from scipy import r_, c_
    a = eval('r_['+m*'a,'+']')
    return eval('c_['+n*'a,'+']')

def char2str(list, format = 'c'):
    import array
    return array.array(format, list).tostring()
#Find: \ +'(\w+)'\)\ *\n\ +(\d+)\ *\n\ +'(\w+)'\)\ *\n\ +(-*\d+),
# Replace: '$1':\($2,$3,$4\),\n

def get_fs(f):
    abfreader = Abf(f)
    d = abfreader.abfload()
    us_interval  = np.ceil(d[1]['si'])
    fs_khz = int((1000.0/us_interval)[0])
    return fs_khz

def get_si(f):
    abfreader = Abf(f)
    d = abfreader.abfload()
    data_dict = d[1]
    si= data_dict['fADCSampleInterval']
    return si

def get_info(f):
    abfreader = Abf(f)
    d = abfreader.abfload()
    data_dict = d[1]


if __name__ == '__main__':

    import os
    fs_dictionary = {}
    count = 0
    basedir0 = '/Volumes/LACIE SHARE/VM_data/All_Data_Jan_2016/PV_Arch/'
    basedir1 = '/Volumes/LACIE SHARE/VM_data/All_Data_Jan_2016/PV_ChR2/'
    basedir2 = '/Volumes/LACIE SHARE/VM_data/All_Data_Jan_2016/Som_ChR2/'
    for basedir in [basedir0,basedir1,basedir2]:
        print '*********', basedir, '*********'
        for name in os.listdir(basedir):
            if name[-4:] == '.abf':
                fs_khz = get_fs(basedir+name)
                key = name.split('.')[0]
                print key, ':', fs_khz, ' khz'

                fs_dictionary[key] = fs_khz
                count += 1
    print 'there are ', count, 'filenames'
    c2 = 0
    for key in fs_dictionary.keys():
        c2 += 1
    print c2,'keys in dict'

    import pickle
    f = open('../pickled_fs_dictionary','wb')
    pickle.dump(fs_dictionary, f)




import h5py
import numpy as np
import matplotlib.pyplot as plt
import time
import sys

class H5Dataset():
    """
    This is to just load up the h5 file converted by the ndf loader
    """
    def __init__(self, h5dataset):
        self.h5dataset = h5dataset
        self.data = None
        self.time = None
        self.features = None
        self.col_labels = None
        self._load_data()

    def _load_data(self):
        for member in self.h5dataset.keys():
            if str(member) == 'data':
                self.data = np.array(self.h5dataset['data'])
            if str(member) == 'time':
                self.time = np.array(self.h5dataset['time'])
            if str(member) == 'features':
                self.features = np.array(self.h5dataset['features'])


    def plot(self):
        print ('Placeholder: Plot method to implement!')
        # have indexing argument...

class H5File():
    def __init__(self, filepath):
        self.filepath = filepath
        with h5py.File(self.filepath, 'r+') as f:
            if sys.version_info < (3,):
                self.attributes = dict(f.attrs.iteritems())
            else:
                self.attributes = dict(f.attrs.items())
            self.attributes['Mcode'] = list(f.keys())[0]

    def __repr__(self):
        return 'Better formatting coming soon... \nAttributes:'+str(self.attributes)

    def __getitem__(self, tid):
        assert tid in self.attributes['t_ids'], 'ERROR: Invalid tid for file'
        with h5py.File(self.filepath, 'r+') as f:
            tid_dataset = H5Dataset(f[self.attributes['Mcode']+'/'+str(tid)])
            group_contents = {}
            group_contents['data'] = tid_dataset.data
            group_contents['time'] = tid_dataset.time
            group_contents['features'] = tid_dataset.features
            try:
                group_contents['col_names'] = f[self.attributes['Mcode']+'/'+str(tid)].attrs['col_names'].astype(str)
            except:
                pass
        return group_contents



import sys
import os
import multiprocessing
import numpy as np
import pandas as pd
from PyQt5 import QtGui#,# uic
from PyQt5.QtCore import QThread, pyqtSignal, Qt, QRect, QTimer
import pyqtgraph as pg
import h5py
import logging
import pickle

from sklearn.ensemble import RandomForestClassifier
from sklearn import metrics

try:
    from context import loading_subwindow, convert_ndf_window, library_subwindow, add_pred_features_subwindow, clf_subwindow
    from context import ndf
except:
    from .context import loading_subwindow, convert_ndf_window, library_subwindow, add_pred_features_subwindow, clf_subwindow
    from .context import ndf

from ndf.h5loader import H5File
from ndf.datahandler import DataHandler, NdfFile
from ndf.classifier import Classifier
'''
except:
    import loading_subwindow, convert_ndf_window, library_subwindow, add_pred_features_subwindow, clf_subwindow
    from pyecog.ndf.h5loader import H5File
    from pyecog.ndf.datahandler import DataHandler, NdfFile
    from pyecog.ndf.classifier import Classifier
'''
# todo : these classes could inherit classes that have signals and slots already made, as you kept the gui element names the same when possible.

class ClfWindow(QtGui.QDialog,clf_subwindow.Ui_ClfManagement):
    ''' For handling the classifier...'''
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.home = '' # default folder that can get set when this is class is called from main window
        self.h5directory  = None
        self.library_path = None
        self.clf = None
        self.worker = None

        self.set_h5_folder.clicked.connect(self.get_h5_folder)
        self.set_library.clicked.connect(self.get_library)
        self.make_classifier.clicked.connect(self.make_classifier_method)
        self.save_classifier.clicked.connect(self.save_classifier_method)
        self.load_classifier.clicked.connect(self.load_classifier_method)
        self.train_clf.clicked.connect(self.train_clf_method)
        self.downsample_bl.textChanged.connect(self.updated_sampling_params)
        self.upsample_s_factor.textChanged.connect(self.updated_sampling_params)
        self.estimate_error.clicked.connect(self.not_done_yet)
        self.run_clf_on_folder.clicked.connect(self.predict_seizures)

    def not_done_yet(self):
        QtGui.QMessageBox.information(self,"Not implemented, lazy!", "Not implemented yet! Jonny has been lazy!")

    def get_library(self):
        self.library_path = QtGui.QFileDialog.getOpenFileName(self, "Pick an annotations file", self.home)[0]
        if self.library_path == '':
            print('No path selected')
            return 0
        self.library_path_display.setText(self.library_path)

    def get_label_counts(self):
        self.counts = pd.Series(np.ravel(self.clf.labels[:])).value_counts().values
        self.label_n_baseline.setText('Library has '+ str(self.counts[0]) +' BL chunks')
        self.label_n_seizures.setText('and '+ str(self.counts[1]) +' Seizure chunks. '+str(np.round((self.counts[1]/self.counts[0])*100, 2)) + '%')

        dwnsample_factor = int(self.downsample_bl.text())
        upsample_factor = int(self.upsample_s_factor.text())
        expected_resample = (int(self.counts[0]/dwnsample_factor),self.counts[1]*upsample_factor)
        self.label_resampled_numbers.setText('Expected resample: ' +str(list(expected_resample)) + '. '+
                                             str(np.round((expected_resample[1]/expected_resample[0])*100, 2)) + '%')
    def updated_sampling_params(self):
        try:
            dwnsample_factor = int(self.downsample_bl.text())
            upsample_factor = int(self.upsample_s_factor.text())
        except ValueError:
            #print('Not valid number entered')
            return 0

        expected_resample = (int(self.counts[0]/dwnsample_factor),self.counts[1]*upsample_factor)
        self.label_resampled_numbers.setText('Expected resample: ' +str(list(expected_resample)) + '. '+
                                             str(np.round((expected_resample[1]/expected_resample[0])*100, 2)) + '%')
    def make_classifier_method(self):
        if self.library_path:
            try:
                self.clf = Classifier(self.library_path)
                QtGui.QMessageBox.information(self, "Not?", "Classifier initialised successfully!")
                self.get_label_counts()
            except:
                QtGui.QMessageBox.information(self, "Not?", "ERROR: Something went wrong making the classifier - is the library ready?")
                return 0
        else:
            QtGui.QMessageBox.information(self, "Not?", "Please choose a valid library path")
            return 0

    def save_classifier_method(self):
        self.clf_path = QtGui.QFileDialog.getSaveFileName(self, "Choose savename", self.home)[0]
        print(self.clf_path)
        self.clf_path = self.clf_path if self.clf_path.endswith('.p') else self.clf_path+'.p'
        if self.clf:
            self.clf.save(fname = self.clf_path)
            self.update_clf_path_display()
        else:
            QtGui.QMessageBox.information(self, "Not?", "No classifier to save")
            return 0

    def load_classifier_method(self):
        temp_clf_path = QtGui.QFileDialog.getOpenFileName(self, "Select pickled classifier to load", self.home)[0]
        if temp_clf_path == '':
            print('No folder selected')
            return 0
        else:
            try:
                with open(temp_clf_path, 'rb') as f:
                    self.clf = pickle.load(f)
            except:
                QtGui.QMessageBox.information(self, "Not?", "ERROR: Classifier loading failed. ")
                with open(temp_clf_path, 'rb') as f:
                    self.clf = pickle.load(f)
                return 0

            self.clf_path = temp_clf_path
            self.update_clf_path_display()
            self.get_label_counts()

    def get_h5_folder(self):
        self.h5directory = QtGui.QFileDialog.getExistingDirectory(self, "Pick a h5 folder", self.home)
        if self.h5directory == '':
            print('No folder selected')
            return 0
        self.update_h5_folder_display()

    def update_clf_path_display(self):
        self.clf_path_display.setText(str(self.clf_path))
        print(pd.Series(np.ravel(self.clf.labels[:])).value_counts().values)

    def update_h5_folder_display(self):
        self.h5_folder_display.setText(str(self.h5directory))

    def train_clf_method(self):
        # boot up a thread here and re-implement the train method of the classifier... maybe with more sensible resampling stuff!
        try:
            if self.worker:
                if self.worker.isRunning():
                    QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
                return 0
        except:
            pass

        self.worker = TrainClassifierThread() # assume the previous finished thread is garbage collected...!
        self.worker.update_progress_label.connect(self.update_progress_label)
        self.worker.SetProgressBar.connect(self.update_progress)
        self.worker.setMaximum_progressbar.connect(self.set_max_bar)
        self.worker.update_label_below.connect( self.update_label_below)
        self.worker.update_label_above2.connect( self.update_label_above2)
        self.worker.finished.connect(self.end_training)

        dwnsample_factor = int(self.downsample_bl.text())
        upsample_factor = int(self.upsample_s_factor.text())
        ntrees = int(self.n_trees.text())
        ncores = self.n_cores.text()
        if ncores == 'all':
            ncores = -1
        else:
            ncores = int(ncores)

        self.worker.set_training_params(self.clf, dwnsample_factor, upsample_factor,ntrees, ncores)
        self.worker.start()

    def end_training(self):
        # todo quit threads properly
        QtGui.QMessageBox.information(self, "Not implemented, lazy!", "You probably want to save the trained classifier...!")
        self.save_classifier_method()
        self.worker.quit()
        self.worker.terminate()
        del self.worker

    def end_prediction(self):
        self.worker.quit()
        self.worker.terminate()
        del self.worker


    def update_label_above2(self, label_string):
        self.progressBar_label_above2.setText(label_string)
    def update_progress_label(self, label_string):
        self.progressBar_lable_above1.setText(label_string)
    def update_label_below(self, label_string):
        self.progressBar_label_below.setText(label_string)
    def set_max_bar(self, signal):
        self.progressBar.setMaximum(int(signal))
    def update_progress(self, signal):
        self.progressBar.setValue(int(signal))

    def predict_seizures(self):
        try:
            if self.worker:
                if self.worker.isRunning():
                    QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
                return 0
        except:
            pass

        if self.h5directory  == None:
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Please choose a h5 folder!")
            self.get_h5_folder()
        if self.clf == None:
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Please choose a trained, pickled classifier!")
            self.load_classifier_method()



        self.worker = PredictSeizuresThread() # assume the previous finished thread is garbage collected...!
        self.worker.update_progress_label.connect(self.update_progress_label)
        self.worker.SetProgressBar.connect(self.update_progress)
        self.worker.setMaximum_progressbar.connect(self.set_max_bar)
        self.worker.update_label_below.connect( self.update_label_below)
        self.worker.update_label_above2.connect( self.update_label_above2)
        self.worker.finished.connect(self.end_prediction)

        h5_folder = self.h5_folder_display.text()
        excel_sheet = QtGui.QFileDialog.getSaveFileName(self,  "make output csv file", self.home)[0]

        self.worker.set_params(self.clf, h5_folder, excel_sheet)
        self.worker.start()

class PredictSeizuresThread(QThread):
    finished = pyqtSignal()
    update_progress_label = pyqtSignal(str)
    SetProgressBar= pyqtSignal(str)
    setMaximum_progressbar= pyqtSignal(str)
    update_label_below = pyqtSignal(str)
    update_label_above2 = pyqtSignal(str)

    def __init__(self):
        QThread.__init__(self)
    def set_params(self, clf, prediction_dir, excel_sheet):
        self.clf = clf
        self.prediction_dir = prediction_dir
        self.excel_output = excel_sheet.split('.')[0]+'.csv'
        if os.path.exists(self.excel_output):
            os.remove(self.excel_output)

    def run(self):
        self.update_label_below.emit('Saving predictions: '+ self.excel_output)
        self.update_progress_label.emit('We are now rolling!')
        files_to_predict = [os.path.join(self.prediction_dir,f) for f in os.listdir(self.prediction_dir) if not f.startswith('.')]
        n_files = len(files_to_predict)
        self.setMaximum_progressbar.emit(str(n_files))
        self.update_label_above2.emit('Looking through '+ str(n_files) + ' files for seizures: ')
        for i,fpath in enumerate(files_to_predict):
            fname = str(os.path.split(fpath)[1])
            try:
                with h5py.File(fpath, 'r+') as f:
                    group = f[list(f.keys())[0]]
                    tid = group[list(group.keys())[0]]
                    pred_features = tid['features'][:]

                logging.info(fname + ' now predicting!: ')
                self.update_progress_label.emit(fname+ ': '+ str(i)+'/'+str(n_files))
                self.SetProgressBar.emit(str(i))

                pred_features = self.clf.cleaner.transform(pred_features)
                pred_y_emitts = self.clf.rf.predict(pred_features)
                logp, path = self.clf.hm_model.viterbi(pred_y_emitts)
                vit_decoded_y = np.array([int(state.name) for idx, state in path[1:]])

                if sum(vit_decoded_y):
                    name_array = np.array([fname for i in range(vit_decoded_y.shape[0])])
                    #print (name_array.shape)
                    pred_sheet = self.clf.make_excel_spreadsheet([name_array,vit_decoded_y])
                    #print(pred_sheet.head())
                    if not os.path.exists(self.excel_output):
                        pred_sheet.to_csv(self.excel_output,index = False)
                    else:
                        with open(self.excel_output, 'a') as f:
                            pred_sheet.to_csv(f, header=False, index = False)


            except KeyError:
                logging.error(str(fname) + ' did not contain any features! Skipping')
                self.update_label_below('ERROR: '+ fname + ' did not contain any features! Skipping')

            else:
                pass
                #print ('no seizures')

        self.update_progress_label.emit('Re - ordering spreadsheet by date')
        self.clf.reorder_prediction_csv(self.excel_output)
        self.update_progress_label.emit('Done')
        self.finished.emit()
        self.exit()

class TrainClassifierThread(QThread):
    # add in signals here
    finished = pyqtSignal()
    update_progress_label = pyqtSignal(str)
    SetProgressBar= pyqtSignal(str)
    setMaximum_progressbar= pyqtSignal(str)
    update_label_below = pyqtSignal(str)
    update_label_above2 = pyqtSignal(str)

    def __init__(self):
        QThread.__init__(self)

    def set_training_params(self, clf, downsample_bl_by_x, upsample_seizure_by_x, ntrees, n_cores):
        # you sort out the re-sampling and n trees here
        # or do you want to use the class importances?
        self.clf = clf
        self.n_cores = n_cores
        self.ntrees = ntrees
        self.downsample_bl_factor    = downsample_bl_by_x
        self.upsample_seizure_factor = upsample_seizure_by_x

        counts = pd.Series(np.ravel(self.clf.labels[:])).value_counts().values
        target_resample = (int(counts[0]/self.downsample_bl_factor),counts[1]*self.upsample_seizure_factor)
        self.update_label_above2.emit('Resampling [BL S] from '+str(counts)+' to ' + str(list(target_resample)))
        self.res_y, self.res_x = self.clf.resample_training_dataset(self.clf.labels, self.clf.features,
                                                      sizes = target_resample)

    def run(self):
        #emit the correct signals
        self.update_progress_label.emit('Training Random Forest...')
        self.clf.rf =  RandomForestClassifier(n_jobs=self.n_cores, n_estimators= self.ntrees, oob_score=True, bootstrap=True)
        self.clf.rf.fit(self.res_x, np.ravel(self.res_y))
        self.update_progress_label.emit('Getting Hidden Markov Model params...')
        self.clf.make_hmm_model() # this has default fold arguement of 3

        print ('********* oob results on resampled data  - not including HMM *******')
        self.oob_preds = np.round(self.clf.rf.oob_decision_function_[:,1])
        print('ROC_AUC score: '+str(metrics.roc_auc_score(np.ravel(self.res_y), self.clf.rf.oob_decision_function_[:,1])))
        print('Recall: '+str(metrics.recall_score(np.ravel(self.res_y), self.oob_preds)))
        print('F1: '+str(metrics.f1_score(np.ravel(self.res_y), self.oob_preds)))
        print(metrics.classification_report(np.ravel(self.res_y),self.oob_preds))

        # would be very nice to emit this back
        #self.feature_weightings = sorted(zip(self.clf.rf.feature_importances_, self.clf.feature_names),reverse = True)

        self.finished.emit()
        self.exit()

class AddPredictionFeaturesWindow(QtGui.QDialog, add_pred_features_subwindow.Ui_make_features):

    ''' Add predictions to h5 folder '''
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.set_h5_folder.clicked.connect(self.get_h5_folder)
        self.extract_features_button.clicked.connect(self.run_pred_feature_extraction)
        self.run_peakdet_checkBox.stateChanged.connect(self.use_peaks_changed)

        self.home = '' # default folder that can get set when this is class is called from main window
        self.h5directory  = None

        self.extraction_thread = ExtractPredictionFeaturesThread()
        self.extraction_thread.set_progress_bar.connect(self.update_progress)
        self.extraction_thread.set_max_progress.connect( self.set_max_bar)
        self.extraction_thread.update_hidden_label.connect( self.update_hidden)
        self.extraction_thread.update_progress_label.connect( self.update_progress_label)

    def get_h5_folder(self):
        self.h5directory = QtGui.QFileDialog.getExistingDirectory(self, "Pick a h5 folder", self.home)
        if self.h5directory == '':
            print('No foldrr selected')
            return 0
        self.update_h5_folder_display()

    def update_h5_folder_display(self):
        self.h5_display.setText(str(self.h5directory))

    def use_peaks_changed(self):
        self.use_peaks_bool = self.run_peakdet_checkBox.isChecked()

    def update_hidden(self, label_string):
        self.hidden_label.setText(label_string)
    def update_progress_label(self, label_string):
        self.progress_bar_label.setText(label_string)
    def set_max_bar(self, signal):
        self.progressBar.setMaximum(int(signal))
    def update_progress(self, signal):
        self.progressBar.setValue(int(signal))

    def run_pred_feature_extraction(self):
        # grab the settings...

        if self.extraction_thread.isRunning():
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
            return 0

        chunk_len   = int(self.chunk_len_box.text())
        ncores = self.cores_to_use.text()
        use_peaks_bool = self.run_peakdet_checkBox.isChecked()

        if ncores == 'all':
            ncores = -1
        else:
            ncores = int(ncores)

        try:
            logfilepath = logging.getLoggerClass().root.handlers[0].baseFilename
            self.logpath_dsplay.setText(str(logfilepath))
        except:
            print('couldnt get logpath')



        self.extraction_thread.set_params_for_extraction(h5_folder=self.h5directory,
                                            timewindow = chunk_len,
                                            run_peakdet_flag = use_peaks_bool,
                                            n_cores=ncores)
        self.extraction_thread.start()



class ExtractPredictionFeaturesThread(QThread):
    set_max_progress = pyqtSignal(str)
    update_hidden_label = pyqtSignal(str)
    set_progress_bar =  pyqtSignal(str)
    update_progress_label = pyqtSignal(str)

    def __init__(self):
        QThread.__init__(self)
        self.handler = DataHandler()

    def set_params_for_extraction(self, h5_folder,
                                        timewindow,
                                        run_peakdet_flag,
                                        n_cores = -1):

        self.handler.parrallel_flag_pred = True
        self.handler.run_pkdet = run_peakdet_flag
        self.handler.twindow = timewindow
        self.files_to_add_features = [f for f in self.handler.fullpath_listdir(h5_folder) if f.endswith('.h5')]
        if n_cores == -1:
            n_cores = multiprocessing.cpu_count()
        self.n_cores = n_cores

        #l = len(self.files_to_add_features)

        self.set_max_progress.emit(str(len(self.files_to_add_features)))
        self.update_hidden_label.emit(str(len(self.files_to_add_features))+' Files to extract features from')

    def run(self):
        pool = multiprocessing.Pool(self.n_cores)

        for i, _ in enumerate(pool.imap(self.handler.add_predicition_features_to_h5_file, self.files_to_add_features), 1):
            self.set_progress_bar.emit(str(i))
            self.update_progress_label.emit('Progress: ' +str(i)+ ' / '+ str(len(self.files_to_add_features)))

        pool.close()
        pool.join()

        self.update_progress_label.emit('Progress: Done')
        self.handler.parrallel_flag_pred = False # really not sure this is needed- just a hangover from the datahandler code?


class LibraryWindow(QtGui.QDialog, library_subwindow.Ui_LibraryManagement):
    ''' this is for the predictions, csv and h5 folder needed '''
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.select_annotations.clicked.connect(self.select_annotations_method)
        self.select_h5_folder.clicked.connect(self.select_h5_folder_method)
        self.new_library.clicked.connect(self.make_new_library)
        self.add_to_library.clicked.connect(self.append_to_library)
        self.set_library.clicked.connect(self.select_library)
        self.clear_library.clicked.connect(self.clear_library_path)
        self.chunk_length.textChanged.connect(self.chunk_len_changed)
        self.add_labels.clicked.connect(self.calculate_labels_for_library)
        self.add_features.clicked.connect(self.calculate_features_for_library)
        self.use_peaks.clicked.connect(self.use_peaks_changed)
        self.use_peaks.stateChanged.connect(self.use_peaks_changed)
        self.overwrite_box.stateChanged.connect(self.overwrite_box_changed)
        self.fs_box.textChanged.connect(self.fs_box_changed)

        self.library_path = None
        self.annotation_path = None
        self.h5_folder_path = None
        self.home = None # inherit default folder
        self.chosen_chunk_length = int(self.chunk_length.text())
        self.annotation_df = None
        self.overwrite_box.setChecked(True)
        self.overwrite_bool = self.overwrite_box.isChecked()
        self.use_peaks_bool = self.use_peaks.isChecked()
        self.fs = int(self.fs_box.text())

        #self.progressBar etc...
        #self.progressBar_label_above2
        self.spawn_worker()

    def spawn_worker(self):
            self.worker = LibraryWorkerThread()
            self.worker.finished.connect(self.worker_finished)
            self.worker.update_progress_label.connect(self.update_progress_label)
            self.worker.SetProgressBar.connect(self.update_progress)
            self.worker.setMaximum_progressbar.connect( self.set_max_bar)
            self.worker.update_label_below.connect( self.update_label_below)
            self.worker.update_label_above2.connect( self.update_label_above2)


    def worker_finished(self):
        print('worker finished! method called - terminating? - needlessly?')
        if self.worker:
            print(self.worker)
        self.spawn_worker()

    def select_annotations_method(self):
        self.annotation_path = QtGui.QFileDialog.getOpenFileName(self, "Pick an annotations file", self.home)[0]
        self.annotations_display.setText(self.annotation_path)
        if self.annotation_path.endswith('.xlsx'):
            self.annotation_df = pd.read_excel(self.annotation_path)
        elif self.annotation_path.endswith('.csv'):
            self.annotation_df = pd.read_csv(self.annotation_path)
        else:
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Please use .csv or .xlsx files!")
        try:
            self.annotation_df.columns = [label.lower() for label in self.annotation_df.columns]
            self.annotation_df.columns  = [label.strip(' ') for label in self.annotation_df.columns]
            assert 'filename' in self.annotation_df
            assert 'start' in self.annotation_df
            assert 'end'   in self.annotation_df
            assert 'transmitter' in self.annotation_df
        except:
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Please make sure annotations file contains at least 'Filename', 'Start', 'End', 'Transmitter'!")

        # here check fileheadings are all good

    def select_h5_folder_method(self):
        self.h5_folder_path = QtGui.QFileDialog.getExistingDirectory(self, "Pick h5 folder", self.home)
        self.h5_folder_display.setText(self.h5_folder_path)

    def chunk_len_changed(self):
        try:
            self.chosen_chunk_length = int(self.chunk_length.text())
            print(self.chosen_chunk_length)
        except ValueError:
            print('Not valid number entered')


    def fs_box_changed(self):
        self.fs = int(self.fs_box.text())

    def use_peaks_changed(self):
        self.use_peaks_bool = self.use_peaks.isChecked()

    def overwrite_box_changed(self):
        self.overwrite_bool = self.overwrite_box.isChecked()

    def select_library(self, default_lib = None):
        if default_lib:
            self.library_path = QtGui.QFileDialog.getOpenFileName(self,  "Choose Library file", default_lib)[0]
        else:
            self.library_path = QtGui.QFileDialog.getOpenFileName(self,  "Choose Library file", self.home)[0]

        self.update_library_path_display()

    def clear_library_path(self):
        self.library_path = ''
        self.update_library_path_display()

    def update_library_path_display(self):
        self.library_path_display.setText(self.library_path)

    def update_label_above2(self, label_string):
        self.progressBar_label_above2.setText(label_string)
    def update_progress_label(self, label_string):
        self.progressBar_lable_above1.setText(label_string)
    def update_label_below(self, label_string):
        self.progressBar_label_below.setText(label_string)
    def set_max_bar(self, signal):
        self.progressBar.setMaximum(int(signal))
    def update_progress(self, signal):
        self.progressBar.setValue(int(signal))

    def make_new_library(self):
        self.library_path = QtGui.QFileDialog.getSaveFileName(self,  "Make new Library file", self.home)[0]
        if not self.library_path.endswith('.h5'):
            try:
                self.library_path = self.library_path.split('.')[0]+'.h5'
            except:
                self.library_path = self.library_path +'.h5'

        self.update_library_path_display()


        if self.worker.isRunning():
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
            return 0
        elif self.worker.isFinished():
            self.worker.set_library_attributes(self.library_path,
                                     self.annotation_df,
                                     self.h5_folder_path,
                                     self.chosen_chunk_length,
                                     self.overwrite_bool,
                                     self.fs)

            self.worker.new_library_mode()
            self.worker.start()
            self.worker.wait()

            print('Worker finished')
        else:
            print('else got called')
            self.worker.set_library_attributes(self.library_path,
                                     self.annotation_df,
                                     self.h5_folder_path,
                                     self.chosen_chunk_length,
                                     self.overwrite_bool,
                                     self.fs)

            self.worker.new_library_mode()
            self.worker.start()
            self.worker.wait()

    def append_to_library(self):
        if self.library_path:
            self.select_library(default_lib=self.library_path)
        else:
            self.select_library()

        if self.library_path == '':
            print ('No library path chosen')
            return 0

        if self.worker.isRunning():
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
            return 0
        #todo catch not having the annotations and h5 files
        elif self.worker.isFinished():
            self.worker.set_library_attributes(self.library_path,
                                     self.annotation_df,
                                     self.h5_folder_path,
                                     self.chosen_chunk_length,
                                     self.overwrite_bool,
                                     self.fs)

            self.worker.append_to_library_mode()
            self.worker.start()
            #self.worker.wait()
            print('Worker finished')
        else: # why do you have this - for running first time off??
            print('else got called in append to library')
            self.worker.set_library_attributes(self.library_path,
                                     self.annotation_df,
                                     self.h5_folder_path,
                                     self.chosen_chunk_length,
                                     self.overwrite_bool,
                                     self.fs)

            self.worker.append_to_library_mode()
            self.worker.start()
            #self.worker.wait()

    def calculate_features_for_library(self):
        if self.library_path is None:
            self.select_library()
            self.update_library_path_display()

        if self.worker.isRunning():
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
            return 0
        elif self.worker.isFinished():
            self.worker.add_features_mode()
            self.worker.set_library_attributes_for_feats(self.library_path, self.chosen_chunk_length, self.overwrite_bool, self.use_peaks_bool)
            self.worker.start()
            print('Worker finished')
        else:
            print('else got called in labels to library')
            self.worker.add_features_mode()
            self.worker.set_library_attributes_for_feats(self.library_path, self.chosen_chunk_length, self.overwrite_bool, self.use_peaks_bool)
            self.worker.start()


    def calculate_labels_for_library(self):
        if self.library_path is None:
            self.select_library()
            self.update_library_path_display()

        if self.worker.isRunning():
            QtGui.QMessageBox.information(self, "Not implemented, lazy!", "Worker thread still running, please wait for previous orders to be finished!")
            return 0
        elif self.worker.isFinished():
            self.worker.add_labels_mode()
            self.worker.set_library_attributes_for_feats(self.library_path, self.chosen_chunk_length, self.overwrite_bool, self.use_peaks_bool)
            self.worker.start()
            print('Worker finished')

        else:
            print('else got called in labels to library')
            self.worker.add_labels_mode()
            self.worker.set_library_attributes_for_feats(self.library_path, self.chosen_chunk_length, self.overwrite_bool, self.use_peaks_bool)
            self.worker.start()

class LibraryWorkerThread(QThread):

    finished = pyqtSignal(str)
    update_progress_label = pyqtSignal(str)
    SetProgressBar= pyqtSignal(str)
    setMaximum_progressbar= pyqtSignal(str)
    update_label_below =pyqtSignal(str)
    update_label_above2 = pyqtSignal(str)


    def __init__(self):
        QThread.__init__(self)
        self.handler = DataHandler()

    def __del__(self):
        #self.wait()
        self.exit()

    def set_library_attributes(self, l_path, a_df, h5_path, timewindow, overwrite_bool, fs):
        self.library_path = l_path
        self.h5_path = h5_path
        self.annotations_df = a_df
        self.t_len = timewindow
        self.overwrite_bool = overwrite_bool
        self.fs = fs

    def set_library_attributes_for_feats(self, l_path, timewindow, overwrite_bool, peaks_bool):
        self.library_path = l_path
        self.t_len = timewindow
        self.overwrite_bool = overwrite_bool
        self.run_peaks_bool = peaks_bool

    def add_labels_mode(self):
        self.labels_or_features = True
        self.add_features = False

    def add_features_mode(self):
        self.labels_or_features = True
        self.add_features = True

    def new_library_mode(self):
        self.appending_to_library = False
        self.labels_or_features = False

    def append_to_library_mode(self):
        self.appending_to_library = True
        self.labels_or_features = False

    def run(self):

        # this is gonna be a bit of a hack...
        if self.labels_or_features == False:
            if self.appending_to_library:
                self.update_progress_label.emit('Progress Bar is Frozen - no biggy')
                self.handler.append_to_seizure_library(df = self.annotations_df,
                                                       file_dir=self.h5_path,
                                                       seizure_library_path=self.library_path,
                                                       overwrite=self.overwrite_bool,
                                                       timewindow=self.t_len, fs=self.fs)
            else:
                #self.emit(pyqtSignal("update_progress_label(QString)"),'Progress Bar is Frozen - no biggy')
                self.update_progress_label.emit('Progress Bar is Frozen - no biggy')
                self.handler.make_seizure_library(df = self.annotations_df,
                                                       file_dir=self.h5_path,
                                                       seizure_library_name=self.library_path,
                                                       overwrite=self.overwrite_bool,
                                                       timewindow=self.t_len, fs=self.fs)
        elif self.labels_or_features == True:
            if not self.add_features:
                #self.emit(pyqtSignal("update_progress_label(QString)"),'Progress Bar is Frozen - no biggy')
                self.update_progress_label.emit('Progress Bar is Frozen - no biggy')
                self.handler.add_labels_to_seizure_library(self.library_path,self.overwrite_bool,self.t_len)
                print('labels done, chunked: '+ str(self.t_len))

            elif self.add_features:
                print('lets do the features')
                #self.emit(pyqtSignal("update_progress_label(QString)"),'Progress Bar is Frozen - no biggy')
                self.update_progress_label.emit('Progress Bar is Frozen - no biggy')
                self.handler.add_features_seizure_library(self.library_path,self.overwrite_bool,self.run_peaks_bool, self.t_len)
                print('features done, chunked: '+ str(self.t_len))
        self.exit() # does this emit the finished signal? dont think so




class ConvertingNDFsWindow(QtGui.QDialog, convert_ndf_window.Ui_convert_ndf_to_h5):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.select_ndf_folder.clicked.connect(self.get_ndf_folder)
        self.set_h5_folder.clicked.connect(self.get_h5_folder)
        self.convert_button.clicked.connect(self.convert_folder)

        #self.transmitter_ids
        self.home = '' # default folder to be inherited
        self.progressBar.setValue(0)
        self.cores_to_use.setText(str(1))
        self.transmitter_ids.setText(str(8))
        #self.h5directory = '/Volumes/G-DRIVE with Thunderbolt/test_h5'
        #self.ndf_folder  = '/Volumes/G-DRIVE with Thunderbolt/test_ndfs'
        #self.h5_display.setText(str(self.h5directory))
        #self.ndf_display.setText(str(self.ndf_folder))


    def get_h5_folder(self):
        self.h5directory = QtGui.QFileDialog.getExistingDirectory(self, "Pick a h5 folder", self.home)
        self.h5_display.setText(str(self.h5directory))

    def get_ndf_folder(self):
        self.ndf_folder = QtGui.QFileDialog.getExistingDirectory(self, 'Select a ndf folder to convert', self.home)
        self.ndf_display.setText(str(self.ndf_folder))

    def convert_folder(self):
        tids = self.transmitter_ids.text()
        fs   = int(self.fs_box.text())
        ncores = self.cores_to_use.text()
        if ncores == 'all':
            ncores = -1
        else:
            ncores = int(ncores)

        if tids != 'all':
            tids = eval('['+tids+']')

        self.converting_thread = ConvertNdfThread()
        self.converting_thread.set_progress_bar.connect(self.update_progress)
        self.converting_thread.set_max_progress.connect( self.set_max_bar)
        self.converting_thread.update_hidden_label.connect( self.update_hidden)
        self.converting_thread.update_progress_label.connect( self.update_progress_label)

        try:
            logfilepath = logging.getLoggerClass().root.handlers[0].baseFilename
            self.logpath_dsplay.setText(str(logfilepath))
        except:
            print('couldnt get logpath')
            #logfilepath = logging.getLoggerClass().root.handlers[0].baseFilename
        try:

            self.converting_thread.convert_ndf_directory_to_h5(ndf_dir=self.ndf_folder,
                                                save_dir=self.h5directory,
                                                tids=tids,
                                                n_cores=ncores,
                                                fs=fs)
            self.converting_thread.start()
        except:
            QtGui.QMessageBox.information(self,"Not implemented, lazy!", "Error!: /n Stop fucking around?! Currently set to re-run, so check errors in terminal")
            self.converting_thread.convert_ndf_directory_to_h5(ndf_dir=self.ndf_folder,
                                                save_dir=self.h5directory,
                                                tids=tids,
                                                n_cores=ncores,
                                                fs=fs)
            self.converting_thread.start()

    def update_hidden(self, label_string):
        self.hidden_label.setText(label_string)
    def update_progress_label(self, label_string):
        self.progress_bar_label.setText(label_string)
    def set_max_bar(self, signal):
        self.progressBar.setMaximum(int(signal))
    def update_progress(self, signal):
        self.progressBar.setValue(int(signal))

class ConvertNdfThread(QThread):

    set_max_progress = pyqtSignal(str)
    update_hidden_label = pyqtSignal(str)
    set_progress_bar =  pyqtSignal(str)
    update_progress_label = pyqtSignal(str)
    def __init__(self):
        QThread.__init__(self)
        self.handler = DataHandler()


    def convert_ndf_directory_to_h5(self,
                                    ndf_dir,
                                    tids = 'all',
                                    save_dir  = 'same_level',
                                    n_cores = -1,
                                    fs = 'auto'):
        """
        Copy from datahandler, this should be a thread?
        """

        self.handler.fs_for_parallel_conversion = fs
        self.files = [f for f in self.handler.fullpath_listdir(ndf_dir) if f.endswith('.ndf')]

        # tids
        if not tids == 'all':
            if not hasattr(tids, '__iter__'):
                tids = [tids]

        self.handler.tids_for_parallel_conversion = tids

        # set n_cores
        if n_cores == -1:
            n_cores = multiprocessing.cpu_count()
        self.n_cores = n_cores
        # Make save directory
        if save_dir  == 'same_level':
            save_dir = ndf_dir+'_converted_h5s'
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        self.handler.savedir_for_parallel_conversion = save_dir


        l = len(self.files)
        #self.emit(pyqtSignal("update_hidden_label(QString)"), str(len(self.files))+' Files for conversion. Transmitters: '+ str(self.handler.tids_for_parallel_conversion))
        #self.emit(pyqtSignal("setMaximum_progressbar(QString)"),str(len(self.files)))
        self.set_max_progress.emit(str(len(self.files)))
        self.update_hidden_label.emit(str(len(self.files))+' Files for conversion. Transmitters: '+ str(self.handler.tids_for_parallel_conversion))
    def run(self):
        pool = multiprocessing.Pool(self.n_cores)

        for i, _ in enumerate(pool.imap(self.handler.convert_ndf, self.files), 1):
            self.set_progress_bar.emit(str(i))
            self.update_progress_label.emit('Progress: ' +str(i)+ ' / '+ str(len(self.files)))
            #self.emit(pyqtSignal("SetProgressBar(QString)"),str(i))
            #self.emit(pyqtSignal("update_progress_label(QString)"),'Progress: ' +str(i)+ ' / '+ str(len(self.files)))
        pool.close()
        pool.join()
        #self.emit(pyqtSignal("update_progress_label(QString)"),'Progress: Done')
        self.update_progress_label.emit('Progress: Done')

class LoadingSubwindow(QtGui.QDialog, loading_subwindow.Ui_Dialog):
    ''' this is for checking out predictions on main gui, csv and h5 folder needed '''
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.set_prediction_file.clicked.connect(self.get_pred_filename)
        self.set_h5_folder.clicked.connect(self.get_h5_folder)

        self.home = '' # default folder to be inherited
        self.predictions_fname = None
        self.h5directory       = None

    def get_h5_folder(self):
        self.h5directory = QtGui.QFileDialog.getExistingDirectory(self, "Pick a h5 folder", self.home)
        print(self.h5directory)
        print(type(self.h5directory))
        self.update_h5_folder_display()

    def update_h5_folder_display(self):
        self.h5_display.setText(str(self.h5directory))

    def update_predictionfile_display(self):
        self.prediction_display.setText(str(self.predictions_fname))

    def get_pred_filename(self):
        self.predictions_fname = QtGui.QFileDialog.getOpenFileName(self, 'Select a predicitons file', self.home)[0]
        self.update_predictionfile_display()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'clf_window.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ClfManagement(object):
    def setupUi(self, ClfManagement):
        ClfManagement.setObjectName("ClfManagement")
        ClfManagement.resize(915, 766)
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        ClfManagement.setFont(font)
        self.library_path_display = QtWidgets.QLineEdit(ClfManagement)
        self.library_path_display.setGeometry(QtCore.QRect(50, 60, 741, 21))
        self.library_path_display.setMinimumSize(QtCore.QSize(500, 0))
        self.library_path_display.setReadOnly(True)
        self.library_path_display.setObjectName("library_path_display")
        self.label = QtWidgets.QLabel(ClfManagement)
        self.label.setGeometry(QtCore.QRect(50, 40, 81, 16))
        self.label.setObjectName("label")
        self.progressBar = QtWidgets.QProgressBar(ClfManagement)
        self.progressBar.setGeometry(QtCore.QRect(50, 590, 751, 23))
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.progressBar_label_below = QtWidgets.QLabel(ClfManagement)
        self.progressBar_label_below.setGeometry(QtCore.QRect(50, 620, 751, 16))
        self.progressBar_label_below.setText("")
        self.progressBar_label_below.setObjectName("progressBar_label_below")
        self.progressBar_lable_above1 = QtWidgets.QLabel(ClfManagement)
        self.progressBar_lable_above1.setGeometry(QtCore.QRect(50, 570, 741, 16))
        self.progressBar_lable_above1.setObjectName("progressBar_lable_above1")
        self.progressBar_label_above2 = QtWidgets.QLabel(ClfManagement)
        self.progressBar_label_above2.setGeometry(QtCore.QRect(50, 550, 751, 16))
        self.progressBar_label_above2.setText("")
        self.progressBar_label_above2.setObjectName("progressBar_label_above2")
        self.set_library = QtWidgets.QPushButton(ClfManagement)
        self.set_library.setGeometry(QtCore.QRect(50, 90, 131, 32))
        self.set_library.setObjectName("set_library")
        self.line_2 = QtWidgets.QFrame(ClfManagement)
        self.line_2.setGeometry(QtCore.QRect(50, 450, 761, 16))
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.line_3 = QtWidgets.QFrame(ClfManagement)
        self.line_3.setGeometry(QtCore.QRect(50, 530, 751, 16))
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.label_5 = QtWidgets.QLabel(ClfManagement)
        self.label_5.setGeometry(QtCore.QRect(320, 10, 251, 31))
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        font.setPointSize(19)
        font.setBold(False)
        font.setWeight(50)
        self.label_5.setFont(font)
        self.label_5.setIndent(-1)
        self.label_5.setObjectName("label_5")
        self.label_6 = QtWidgets.QLabel(ClfManagement)
        self.label_6.setGeometry(QtCore.QRect(310, 420, 301, 31))
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        font.setPointSize(19)
        font.setBold(False)
        font.setWeight(50)
        self.label_6.setFont(font)
        self.label_6.setIndent(-1)
        self.label_6.setObjectName("label_6")
        self.clf_path_display = QtWidgets.QLineEdit(ClfManagement)
        self.clf_path_display.setGeometry(QtCore.QRect(50, 150, 741, 21))
        self.clf_path_display.setMinimumSize(QtCore.QSize(500, 0))
        self.clf_path_display.setReadOnly(True)
        self.clf_path_display.setObjectName("clf_path_display")
        self.label_2 = QtWidgets.QLabel(ClfManagement)
        self.label_2.setGeometry(QtCore.QRect(50, 130, 111, 16))
        self.label_2.setObjectName("label_2")
        self.make_classifier = QtWidgets.QPushButton(ClfManagement)
        self.make_classifier.setGeometry(QtCore.QRect(180, 90, 131, 32))
        self.make_classifier.setObjectName("make_classifier")
        self.save_classifier = QtWidgets.QPushButton(ClfManagement)
        self.save_classifier.setGeometry(QtCore.QRect(50, 180, 131, 32))
        self.save_classifier.setObjectName("save_classifier")
        self.load_classifier = QtWidgets.QPushButton(ClfManagement)
        self.load_classifier.setGeometry(QtCore.QRect(180, 180, 131, 32))
        self.load_classifier.setObjectName("load_classifier")
        self.label_3 = QtWidgets.QLabel(ClfManagement)
        self.label_3.setGeometry(QtCore.QRect(60, 470, 61, 16))
        self.label_3.setObjectName("label_3")
        self.h5_folder_display = QtWidgets.QLineEdit(ClfManagement)
        self.h5_folder_display.setGeometry(QtCore.QRect(120, 470, 671, 21))
        self.h5_folder_display.setMinimumSize(QtCore.QSize(500, 0))
        self.h5_folder_display.setReadOnly(True)
        self.h5_folder_display.setObjectName("h5_folder_display")
        self.set_h5_folder = QtWidgets.QPushButton(ClfManagement)
        self.set_h5_folder.setGeometry(QtCore.QRect(50, 500, 131, 32))
        self.set_h5_folder.setObjectName("set_h5_folder")
        self.run_clf_on_folder = QtWidgets.QPushButton(ClfManagement)
        self.run_clf_on_folder.setGeometry(QtCore.QRect(180, 500, 131, 32))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.run_clf_on_folder.setFont(font)
        self.run_clf_on_folder.setObjectName("run_clf_on_folder")
        self.train_clf = QtWidgets.QPushButton(ClfManagement)
        self.train_clf.setGeometry(QtCore.QRect(540, 320, 131, 32))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.train_clf.setFont(font)
        self.train_clf.setObjectName("train_clf")
        self.line_4 = QtWidgets.QFrame(ClfManagement)
        self.line_4.setGeometry(QtCore.QRect(50, 341, 761, 16))
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.label_7 = QtWidgets.QLabel(ClfManagement)
        self.label_7.setGeometry(QtCore.QRect(50, 320, 251, 31))
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        font.setPointSize(19)
        font.setBold(False)
        font.setWeight(50)
        self.label_7.setFont(font)
        self.label_7.setIndent(-1)
        self.label_7.setObjectName("label_7")
        self.estimate_error = QtWidgets.QPushButton(ClfManagement)
        self.estimate_error.setGeometry(QtCore.QRect(50, 361, 131, 32))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.estimate_error.setFont(font)
        self.estimate_error.setObjectName("estimate_error")
        self.cv_nfolds = QtWidgets.QLineEdit(ClfManagement)
        self.cv_nfolds.setGeometry(QtCore.QRect(310, 361, 31, 21))
        self.cv_nfolds.setObjectName("cv_nfolds")
        self.label_4 = QtWidgets.QLabel(ClfManagement)
        self.label_4.setGeometry(QtCore.QRect(200, 361, 101, 20))
        self.label_4.setObjectName("label_4")
        self.error_label = QtWidgets.QLabel(ClfManagement)
        self.error_label.setGeometry(QtCore.QRect(370, 360, 441, 31))
        self.error_label.setText("")
        self.error_label.setObjectName("error_label")
        self.n_cores = QtWidgets.QLineEdit(ClfManagement)
        self.n_cores.setGeometry(QtCore.QRect(590, 190, 31, 21))
        self.n_cores.setObjectName("n_cores")
        self.label_8 = QtWidgets.QLabel(ClfManagement)
        self.label_8.setGeometry(QtCore.QRect(500, 190, 91, 20))
        self.label_8.setObjectName("label_8")
        self.downsample_bl = QtWidgets.QLineEdit(ClfManagement)
        self.downsample_bl.setGeometry(QtCore.QRect(590, 220, 31, 21))
        self.downsample_bl.setObjectName("downsample_bl")
        self.label_9 = QtWidgets.QLabel(ClfManagement)
        self.label_9.setGeometry(QtCore.QRect(470, 220, 111, 20))
        self.label_9.setObjectName("label_9")
        self.label_10 = QtWidgets.QLabel(ClfManagement)
        self.label_10.setGeometry(QtCore.QRect(450, 250, 121, 21))
        self.label_10.setObjectName("label_10")
        self.upsample_s_factor = QtWidgets.QLineEdit(ClfManagement)
        self.upsample_s_factor.setGeometry(QtCore.QRect(590, 250, 31, 21))
        self.upsample_s_factor.setObjectName("upsample_s_factor")
        self.BL_weight = QtWidgets.QLineEdit(ClfManagement)
        self.BL_weight.setGeometry(QtCore.QRect(710, 220, 51, 21))
        self.BL_weight.setObjectName("BL_weight")
        self.s_weight = QtWidgets.QLineEdit(ClfManagement)
        self.s_weight.setGeometry(QtCore.QRect(710, 250, 51, 21))
        self.s_weight.setObjectName("s_weight")
        self.label_11 = QtWidgets.QLabel(ClfManagement)
        self.label_11.setGeometry(QtCore.QRect(690, 190, 121, 21))
        self.label_11.setObjectName("label_11")
        self.label_12 = QtWidgets.QLabel(ClfManagement)
        self.label_12.setGeometry(QtCore.QRect(680, 220, 21, 21))
        self.label_12.setObjectName("label_12")
        self.label_13 = QtWidgets.QLabel(ClfManagement)
        self.label_13.setGeometry(QtCore.QRect(680, 250, 21, 21))
        self.label_13.setObjectName("label_13")
        self.label_n_seizures = QtWidgets.QLabel(ClfManagement)
        self.label_n_seizures.setGeometry(QtCore.QRect(50, 240, 381, 21))
        self.label_n_seizures.setText("")
        self.label_n_seizures.setObjectName("label_n_seizures")
        self.label_n_baseline = QtWidgets.QLabel(ClfManagement)
        self.label_n_baseline.setGeometry(QtCore.QRect(50, 220, 371, 31))
        self.label_n_baseline.setText("")
        self.label_n_baseline.setObjectName("label_n_baseline")
        self.n_trees = QtWidgets.QLineEdit(ClfManagement)
        self.n_trees.setGeometry(QtCore.QRect(640, 280, 31, 21))
        self.n_trees.setObjectName("n_trees")
        self.label_14 = QtWidgets.QLabel(ClfManagement)
        self.label_14.setGeometry(QtCore.QRect(570, 280, 61, 21))
        self.label_14.setObjectName("label_14")
        self.line = QtWidgets.QFrame(ClfManagement)
        self.line.setGeometry(QtCore.QRect(430, 180, 20, 121))
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.line_5 = QtWidgets.QFrame(ClfManagement)
        self.line_5.setGeometry(QtCore.QRect(790, 180, 20, 121))
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.line_6 = QtWidgets.QFrame(ClfManagement)
        self.line_6.setGeometry(QtCore.QRect(450, 300, 331, 20))
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.line_7 = QtWidgets.QFrame(ClfManagement)
        self.line_7.setGeometry(QtCore.QRect(450, 170, 331, 20))
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.label_resampled_numbers = QtWidgets.QLabel(ClfManagement)
        self.label_resampled_numbers.setGeometry(QtCore.QRect(50, 280, 321, 31))
        self.label_resampled_numbers.setText("")
        self.label_resampled_numbers.setObjectName("label_resampled_numbers")

        self.retranslateUi(ClfManagement)
        QtCore.QMetaObject.connectSlotsByName(ClfManagement)

    def retranslateUi(self, ClfManagement):
        _translate = QtCore.QCoreApplication.translate
        ClfManagement.setWindowTitle(_translate("ClfManagement", "Dialog"))
        self.label.setText(_translate("ClfManagement", "Library Path:"))
        self.progressBar_lable_above1.setText(_translate("ClfManagement", "Progress"))
        self.set_library.setText(_translate("ClfManagement", "Set Library"))
        self.label_5.setText(_translate("ClfManagement", "Make classifier from library"))
        self.label_6.setText(_translate("ClfManagement", "Apply classifier to a h5 folder"))
        self.label_2.setText(_translate("ClfManagement", "Classifier Path:"))
        self.make_classifier.setText(_translate("ClfManagement", "Intialise Classifier"))
        self.save_classifier.setText(_translate("ClfManagement", "Save Classifier"))
        self.load_classifier.setText(_translate("ClfManagement", "Load Classifier"))
        self.label_3.setText(_translate("ClfManagement", "h5 folder:"))
        self.set_h5_folder.setText(_translate("ClfManagement", "Choose h5 folder"))
        self.run_clf_on_folder.setText(_translate("ClfManagement", "Run clf on folder"))
        self.train_clf.setText(_translate("ClfManagement", " Train Classifier"))
        self.label_7.setText(_translate("ClfManagement", "Estimate error"))
        self.estimate_error.setText(_translate("ClfManagement", " Estimate Error"))
        self.cv_nfolds.setText(_translate("ClfManagement", "3"))
        self.label_4.setText(_translate("ClfManagement", "Cross Val n folds:"))
        self.n_cores.setText(_translate("ClfManagement", "all"))
        self.label_8.setText(_translate("ClfManagement", "Set n cores:"))
        self.downsample_bl.setText(_translate("ClfManagement", "4"))
        self.label_9.setText(_translate("ClfManagement", "Downsample BL:"))
        self.label_10.setText(_translate("ClfManagement", "Upsample seizures:"))
        self.upsample_s_factor.setText(_translate("ClfManagement", "1"))
        self.BL_weight.setText(_translate("ClfManagement", "auto"))
        self.s_weight.setText(_translate("ClfManagement", "auto"))
        self.label_11.setText(_translate("ClfManagement", "Class weights"))
        self.label_12.setText(_translate("ClfManagement", "BL"))
        self.label_13.setText(_translate("ClfManagement", "S"))
        self.n_trees.setText(_translate("ClfManagement", "800"))
        self.label_14.setText(_translate("ClfManagement", "No. Trees"))

